#!/usr/bin/python3
"""

Search untranslated strings

"""
# Copyright (C) 2015, Canonical Ltd (http://www.canonical.com/)
#
# Authors:
#    Jean-Baptiste Lallement <jean-baptiste.lallement@canonical.com>
#
# This software is free software: you can redistribute it
# and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# This software is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software.  If not, see <http://www.gnu.org/licenses/>.
#
import os
import logging
from pprint import pprint, pformat
import sys
import subprocess
import apt
import apt_pkg
from distutils.dir_util import copy_tree
import configparser
import re


class image(object):
    def __init__(self, image_dir, update=True):
        """ Constructor """
        self.image_dir = image_dir
        self.update = update
        self.root_dir = os.path.join(image_dir, 'image/system')

        self.cache = None
        self.src_cache_dir = ""

        self.init_version()
        self.setup()
        self.installed_packages = self.list_installed_packages()
        self.installed_languages = self.list_installed_languages()
        logging.debug('Languages: %s', self.installed_languages)

    def setup(self):
        """ Update package lists """

        self.copy_apt_methods()
        apt_pkg.init()

        conf = apt_pkg.config
        conf.clear('Apt::Update')

        conf.set("RootDir", self.root_dir)
        conf.set("APT::Architecture", "armhf")
        conf.set("APT::Architectures::", "armhf")

        self.cache = apt.Cache()
        self.cache.open()
        if self.update:
            logging.info('Updating package cache')
            self.cache.update()

    def init_version(self):
        channel = os.path.join(self.root_dir, 'etc/system-image/channel.ini')
        config = configparser.ConfigParser()
        config.read(channel)
        self.channel = config['service']['channel']
        self.device = config['service']['device']
        self.build_number = config['service']['build_number']
        logging.info('Image: %s %s %s',
                     self.channel,
                     self.device,
                     self.build_number)

    def copy_apt_methods(self):
        """ Copy the local apt methods to the image directory """
        methods_dir = 'usr/lib/apt/methods'
        logging.debug('Copying apt methods')
        copy_tree(
            os.path.join('/', methods_dir),
            os.path.join(self.root_dir, methods_dir)
        )

    def list_installed_packages(self):
        """ list all installed packages """
        admin_dir = os.path.join(self.root_dir, 'var/lib/dpkg')
        try:
            logging.debug('Retrieving list of installed packages')
            out = subprocess.check_output(['dpkg', '--admindir', admin_dir,
                                           '--get-selections'])
        except subprocess.CalledProcessError as exc:
            logging.error('An error occurred while fetching the list of '
                          ' installed packages: %s', exc)
            return None

        # Select installed packages and strip the architecture from the name
        return [pkg.split('\t', 1)[0].split(':', 1)[0]
                for pkg in out.decode('utf-8').split('\n')
                if pkg.endswith('\tinstall')]

    def list_installed_languages(self):
        """ Returns the list of languages installed on the platform """
        languages = []
        langpack_dir = os.path.join(self.root_dir,
                                    'usr/share/locale-langpack')
        languages = [f for f in os.listdir(langpack_dir) if
                     os.path.isdir(os.path.join(langpack_dir, f))]
        return languages

    def fetch_source_packages(self, packages="", force_update=False):
        """ Retrieve source pacakges for a list of binary packages """
        self.src_cache_dir = os.path.join(
            os.path.expanduser('~/.cache/i18_check'), self.channel)

        if not (self.update or force_update):
            logging.info('Update set to false, not fetching source packages')
            return

        # Create source cache directory
        self.src_cache_dir = os.path.join(
            os.path.expanduser('~/.cache/i18_check'), self.channel)
        if not os.path.exists(self.src_cache_dir):
            logging.info('Create source packages cache directory: %s',
                         self.src_cache_dir)
            os.makedirs(self.src_cache_dir)

        logging.info('Fetching %d packages', len(self.installed_packages))

        patterns = self._load_packages(packages)
        for pkg in self.installed_packages:
            skip_pkg = True
            for pattern in patterns:
                if re.match(pattern, pkg):
                    logging.debug("Adding '%s'. Matches pattern '%s'.", pkg, pattern)
                    skip_pkg = False
                    break
            if skip_pkg:
                logging.debug("Skipping '%s'. Doesn't matches any pattern", pkg)
                continue
            # REMOVE ME
            #continue
            p = self.cache[pkg]
            candidate = p.candidate
            dsc_path = os.path.join(self.src_cache_dir,
                                    "{}_{}.dsc".format(candidate.source_name,
                                                       candidate.source_version))
            logging.debug('dsc path: %s', dsc_path)
            if os.path.exists(dsc_path):
                logging.info('Source package %s %s already downloaded. '
                             'Skipping.',
                             candidate.source_name,
                             candidate.source_version)
                continue

            logging.info("Fetching source package of %s", candidate)
            try:
                extract_dir = candidate.fetch_source(self.src_cache_dir)
            except ValueError as exc:
                logging.error('Downloading of %s failed: %s', candidate, exc)
            except subprocess.CalledProcessError as exc:
                logging.error('fetch_source failed: %s', exc)

    def _load_packages(self, path):
        """ Load a list of patterns fom a config file """
        patterns = []
        if not os.path.exists(path):
            return patterns

        logging.debug('Loading list of packages from %s', path)
        with open(path, 'r') as f:
            content = f.readlines()
        patterns = [p.strip() for p in content]
        return patterns
