#!/usr/bin/python3
"""

Search untranslated strings

"""
# Copyright (C) 2014-2015, Canonical Ltd (http://www.canonical.com/)
#
# Authors:
#    Jean-Baptiste Lallement <jean-baptiste.lallement@canonical.com>
#
# This software is free software: you can redistribute it
# and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# This software is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software.  If not, see <http://www.gnu.org/licenses/>.
#
import os
import sys
import logging
from pprint import pprint, pformat
import subprocess
try:
    import polib
except ImportError:
    print('Please install python3-polib')
    sys.exit(1)


CLICK_DIRS = ['custom/click',
              'opt/click.ubuntu.com',
              'usr/share/click/preinstalled']

GETTEXT_CMD = "xgettext --c++ --qt --keyword=tr --keyword=tr:1,2 " \
        "--keyword=N_ --keyword=ctr:1c,2 --from-code=utf-8 -o {}"


def get_translatable_strings(dir):
    """ Load translatable strings from files
        Only QML is supported at the moment
    """

    filelist = find_files_by_extension(dir, ".qml")
    # logging.debug(pformat(filelist))


def find_files_by_extension(dir, ext):
    filelist = []
    for root, dirs, files in os.walk(dir):
        for file in files:
            if file.endswith(ext):
                filelist.append(os.path.join(root, file))

    return filelist


def parse_qml_file_for_translation(path):
    """ Parse a file for translatable strings """
    pass


def check_translations(image_dir, src_dir, locales, type=None):
    if type is None or 'click' in type:
        check_click(image_dir, locales)

    if type is None or 'deb' in type:
        check_deb(src_dir, locales)


def check_click(dir, locales):
    logging.debug('Checking translation for click packages in %s', dir)
    report = {}

    for pkg in get_click_packages(dir):
        report[pkg] = {}
        pkgname = os.path.basename(pkg)
        print('= Package: %s =' % pkgname)
        logging.debug('Checking translations for package %s', pkgname)
        pofile = os.path.join(dir, "{}.po".format(os.path.basename(pkg)))
        if generate_po(pkg, pofile):
            if not os.path.isfile(pofile):
                logging.warning("W: No po file: %s", pofile)
                report[pkg] = {'error': 'No po file: %s' % pofile}
                continue

            for locale in locales:
                report[pkg][locale] = {}
                print('== Locale: %s ==' % locale)
                mofiles = locate_mo(pkg, locale)
                if not mofiles:
                    logging.warning("W: No mo file found")
                    report[pkg][locale] = {'error': 'No mo file found'}
                    continue

                (strcnt, untranscnt, untranslated) = find_untranslated_strings(pofile, mofiles)
                report[pkg][locale] = [strcnt, untranscnt, untranslated]

#                percent = 0
#                if strcnt > 0:
#                    percent = 100.00 * (strcnt-untranscnt)/strcnt
#                print('-> %d / %d - %.2f %%' % (untranscnt, strcnt,percent))
#                if untranslated:
#                    for entry in untranslated:
#
#                        print('*****')
#                        print(po_comment(entry))
#                        print(entry.msgid)
#                        print('*****')
        else:
            report[pkg] = {'error': 'No qml file found'}
        # find_untranslatable_strings(pkg)
    do_report(report)


def do_report(report, file=None):
    for pkg, locales in sorted(report.items()):
        pkgname = os.path.basename(pkg)
        print('= %s =' % pkgname)
        if 'error' in locales:
            print('Error: %s' % locales['error'])
        else:
            for locale, messages in sorted(locales.items()):
                print('== %s ==' % locale)
                pprint(messages)
                try:
                    if messages[2]:
                        for entry in messages[2]:
                            print('*****')
                            print(po_comment(entry))
                            print(entry.msgid)
                            print('*****')
                except KeyError:
                    pass


def po_comment(entry):
    return '\n'.join(
        [line[3:] for line in str(entry).splitlines() if line.startswith('#: ')])


def check_deb(dir, locales):
    logging.debug('Checking translation for deb packages in %s', dir)

    for pkg in next(os.walk(dir))[1]:
        logging.debug('= Package: %s =', pkg)
        pofile = os.path.join(dir, "{}.po".format(os.path.basename(pkg)))

def get_click_packages(dir):
    """ Returns a list of click packages """
    pkgs = set()
    for path in CLICK_DIRS:
        clickpath = os.path.join(dir, path)
        pkgs |= set([os.path.join(clickpath, f) for f in os.listdir(clickpath) if
                    os.path.isdir(os.path.join(clickpath, f))])
    return pkgs


def generate_po(path, outputfile):
    """ Generate a po file """
    logging.debug('Generating %s from %s', outputfile, path)
    filelist = find_files_by_extension(path, ".qml")
    # logging.debug(pformat(filelist))

    if filelist:
        cmd = GETTEXT_CMD.format(outputfile).split()
        cmd.extend(filelist)
        logging.debug('Executing: %s', " ".join(cmd))
        subprocess.check_call(cmd)
    else:
        print('W: No QML files found')
        return False
    return True


def locate_mo(path, locale):
    """ Find mo files in a source tree """
    filelist = find_files_by_extension(path, ".mo")
    return [f for f in filelist if '/{}/LC_MESSAGES/'.format(locale) in f]


def find_untranslated_strings(pofile, mofiles):
    """ Returns a list of unstranslated strings """
    untranslated = []
    if not mofiles:
        print("W: No mo file found")
        return 0, 0, untranslated

    if not os.path.isfile(pofile):
        print("W: po file doesn't exist: %s", pofile)
        return 0, 0, untranslated

    mofile = mofiles[0]
    po = polib.pofile(pofile, encoding='utf-8')

    mo = polib.mofile(mofile, encoding='utf-8')
    t = mo.translated_entries()

    # Merge all the mo files, it might not be entirely accurate as only one
    # MO is used
    if len(mofiles) > 1:
        for mofile in mofiles:
            mo = polib.mofile(mofile, encoding='utf-8')
            t += mo.translated_entries()

    for entry in po:
        if (entry not in t):
            untranslated.append(entry)
    return len(po), len(untranslated), untranslated


def find_untranslatable_strings(pofile, mofile):
    """ Returns a list of unstranslatable strings """
    pass
