#!/usr/bin/python3
"""

Search untranslated strings

"""
# Copyright (C) 2015, Canonical Ltd (http://www.canonical.com/)
#
# Authors:
#    Jean-Baptiste Lallement <jean-baptiste.lallement@canonical.com>
#
# This software is free software: you can redistribute it
# and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# This software is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software.  If not, see <http://www.gnu.org/licenses/>.
#
import os
import logging
from pprint import pprint, pformat
import re
import sys
import subprocess
import json
import codecs
from urllib.request import urlopen

CLICKS_TAR = "clicks.tgz"
CMD_FIND_CLICKS = 'find /custom/click/ /opt/click.ubuntu.com/ -type f '\
        '\( -name "*.qml" -o -name "*.mo" \) -print0 | tar czf /tmp/%s --null '\
        '-T -' % CLICKS_TAR

SYSTEM_IMAGE_URL = "http://system-image.ubuntu.com/"
CACHEDIR = os.path.expanduser("~/.cache/ubuntuimages/")


def get_files_from_device(target, method=None, type=None, target_dir='/tmp'):
    if method.lower() == 'ssh':
        get_files_with_ssh(target, target_dir)
    elif method.lower() == 'adb':
        get_files_with_adb(target, target_dir)
    else:
        logging.error('Unknown connection type: %s', method)
        sys.exit(1)

    untar_cmd = "tar xzf %s -C %s" % (os.path.join(target_dir, CLICKS_TAR),
                                      target_dir)
    logging.debug('Unpacking archive to %s', target_dir)
    subprocess.check_call(untar_cmd.split())


def get_files_with_ssh(hostname, target_dir='/tmp/'):
    """ Retrieve files from the phone over ssh """
    logging.debug('Fetching files from %s over ssh', hostname)
    logging.info('Creating archive')
    subprocess.check_call(['ssh', hostname, CMD_FIND_CLICKS])
    logging.info('Copying archive to %s', target_dir)
    subprocess.check_call(['scp', '-r',
                           '%s:/tmp/%s' % (hostname, CLICKS_TAR),
                           target_dir])


def get_files_with_adb(serial=None, target_dir='/tmp/'):
    """ Retrieve files from the phone over adb """
    logging.debug('Fetching files from device over adb')


def get_parts_ids(channel, build=None):
    """ Returns a tuple of parts that make a phone filesystem
    (ubuntu,device,custom)
    """
    json_url = "%s%s/index.json" % (SYSTEM_IMAGE_URL, channel)
    logging.debug('Loading data from %s', json_url)
    response = urlopen(json_url)
    if response.getcode() != 200:
        logging.error('An error %d occured while fetching data',
                      response.getcode())
        return None

    reader = codecs.getreader('utf-8')
    data = json.load(reader(response))

    ubuntu_part = None
    device_part = None
    custom_part = None
    version_part = None
    version = None

    if 'images' in data:
        for image in data['images']:
            if image['type'] != 'full':
                continue
            if build:
                if int(build) == image['version']:
                    version = image['version']
                    ubuntu_part = image['files'][0]['path']
                    device_part = image['files'][1]['path']
                    custom_part = image['files'][2]['path']
                    version_part = image['files'][3]['path']
                    return (version, ubuntu_part, device_part, custom_part,
                            version_part)
            else:
                version = image['version']
                ubuntu_part = image['files'][0]['path']
                device_part = image['files'][1]['path']
                custom_part = image['files'][2]['path']
                version_part = image['files'][3]['path']

        if not version:
            logging.warning('Build not found for revision %s', build)
        return (version, ubuntu_part, device_part, custom_part, version_part)
    else:
        return (None, None, None, None, None)


def download_parts(channel, device, version):
    """ Download device parts if not already cached """
    if not version:
        logging.error('Please provide a build number.')
        return False
    logging.debug('Downloading image version #%d', version)

    # Run ubuntu-device-flash
    cmd = 'ubuntu-device-flash --download-only --revision {revision:d} ' \
          'touch --channel {channel} --device {device}'.format(
                revision=version,
                channel=channel,
                device=device)
    logging.debug('Executing %s', cmd)

    try:
        logging.info("Downloading %s %s %s", channel, device, version)
        subprocess.check_call(cmd.split())
    except subprocess.CalledProcessError as exc:
        logging.error("ubuntu-device-flash failed with: %s", exc)
        return False
    return True


def extract_tarballs(target_dir, ubuntu, device, custom, version):
    """ Extract the ubuntu and custom tarballs """
    cmd = "tar xJf {path} -C {target} --exclude system/dev/*"
    target_dir = os.path.join(target_dir, 'image')
    os.makedirs(target_dir)
    for path in (ubuntu, custom, version):
        # Remove leading /
        if path.startswith('/'):
            path = path[1:]
        path = os.path.join(CACHEDIR, path)
        if not os.path.exists(path):
            logging.error('File does not exist: %s', path)
            return False

        logging.info('Extracting %s', path)
        try:
            logging.debug(cmd.format(path=path, target=target_dir).split())
            subprocess.check_call(cmd.format(path=path,
                                             target=target_dir).split())
        except subprocess.CalledProcessError as exc:
            logging.error('Extraction failed: %s', exc)
            return False
    return True
