# qakit - tools for UEQA engineers
# Copyright (C) 2014 Thomi Richards
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


from configparser import RawConfigParser
import os.path
import sys
import csv
import requests
from pprint import pprint
import bleach


ACCOUNT_URL = 'https://canonical.leankit.com/Kanban/Api/'
BOARD_ID = '109656104'


def project_backlog():
    try:
        username, password = get_lkkb_auth_pair()
        resp = requests.get(
            ACCOUNT_URL + 'boards/' + BOARD_ID,
            auth=(username, password)
        )
        resp.raise_for_status()
        data = resp.json()
        (sorted_lane,) = [
            l for l in data[u'ReplyData'][0][u'Backlog'] if l[u'Title'] == u'Sorted'
        ]

        row_keys = [
            'Title',
            'Priority',
            'PriorityText',
            'Tags',
            'Description',
        ]
        csv_writer = csv.writer(sys.stdout)
        csv_writer.writerow(row_keys)
        for card in sorted_lane['Cards']:
            row = [ clean_text(card[k]) for k in row_keys]
            csv_writer.writerow(row)
    except IOError:
        handle_file_read_error()


def clean_text(text):
    text = bleach.clean(
        text,
        tags=[],
        attributes=[],
        strip=True,
        strip_comments=True
    )
    text = text.replace('\n', '').replace('\r', '').replace('\t', '')
    return text.encode('utf-8')


def stripIt(string, tag):
    soup = BeautifulSoup(string)

    rmtags = soup.findAll(tag)
    for t in rmtags:
        string = string.replace(str(t), '')
    return string


def get_lkkb_auth_pair(config_file_path=None):
    """Return a tuple of (username, password) from the users config file.

    Raises IOError if the file is not found.
    Raises ValueError if the required configuration is not set.
    """
    if config_file_path is None:
        config_file_path = os.path.expanduser('~/config/qakit.ini')
    parser = RawConfigParser()
    if not parser.read(config_file_path):
        raise IOError("Could not read file %s" % (config_file_path))
    if not parser.has_section("LeanKit"):
        raise ValueError("Could not find 'LeanKit' section in config file.")
    if not parser.has_option("LeanKit", "Username") or not parser.get("LeanKit", "Username"):
        raise ValueError("No username set in config file.")
    if not parser.has_option("LeanKit", "Password") or not parser.get("LeanKit", "Password"):
        raise ValueError("No password set in config file.")
    return (
        parser.get("LeanKit", "Username"),
        parser.get("LeanKit", "Password"),
    )


def handle_file_read_error():
    path = get_default_config_file_path()
    os.makedirs(os.path.dirname(path))
    with open(path, 'w') as f:
        f.write(
            "[LeanKit]\n"
            "# Fill in your LeanKit username and password here:\n"
            "Username = \n"
            "Password = \n"
        )
    # TODO: Make this file readable by this user only!
    print("Missing configuration details!")
    print("A default configuration file has been written to:")
    print("\t%s" % path)
    print("Please fill in the values in that file and re-run this command.")
    sys.exit(2)


def get_default_config_file_path():
    return os.path.expanduser('~/config/qakit.ini')
