#!/usr/bin/python3
# UESQA Metrics
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import argparse
import configparser
import datetime
import dateutil.parser
import logging
import sys

import pymongo

import qakit.config as qakit_config
import qakit.metrics.trello as trello
import qakit.trello as trelloapi
import qakit.metrics.util as util

logger = logging.getLogger(__name__)


def upsert_silo(trello_db, silo):
    """Insert a silo into our db.

    :param trello_db: pymongo.MongoClient db for silos
    :param silo: computed silo

    """
    try:
        return trello_db.silos.update(
            {'id': silo['id']},
            silo,
            upsert=True)
    except pymongo.errors.PyMongoError as e:
        logger.error(e)


def get_card_created_action(trello_db, card_id):
    """Return the 'action' by which the given card was created.

    'Actions' come from the Trello API where they're represented in
    JSON; we store them in our db unmodified.  Actions have types,
    users, data, etc. to describe all the actions possible by a user
    in the Trello UI.  Here we return the card's action of type
    'createCard'--worth noting that Trello only returns modifications
    and comments by default.

    :param trello_db: pymongo.MongoClient db for Trello items
    :param card_id: Trello card id string
    :raises ValueError: if card creation action not found--this can 
        happen if our database begins after this card was created, e.g.

    """
    created_action = trello_db.actions.find_one(
        {'data.card.id': card_id,
         'type': 'createCard'})
    if not created_action:
        # this could happen if our db has an incomplete set of
        # actions, e.g. starting after this card was created
        raise ValueError(
            "Create action for card {} not found.".format(card_id))
    return created_action


def get_card_created_datetime(trello_db, card_id):
    """Return the datetime of a card's 'createCard' action.

    :param trello_db: pymongo.MongoClient db for Trello items
    :param card_id: Trello card id string

    """
    try:
        created_action = get_card_created_action(trello_db, card_id)
    except ValueError:
        # this could happen if our db has an incomplete set of
        # actions, e.g. starting after this card was created
        logger.info(
            "Create action for card {} not found.".format(card_id))
        return None
    return dateutil.parser.parse(created_action['date'])


def get_card_result_action(trello_db, card_id):
    """Return the 'action' of moving to 'Passed' or 'Failed' column.

    'Actions' come from the Trello API where they're represented in
    JSON; we store them in our db unmodified.  Actions have types,
    users, data, etc. to describe all the actions possible by a user
    in the Trello UI.  We're most interested in the card-moving
    actions which result in a card being moved from one list ("under
    testing") to another ("Passed" or "Failed").

    Silo cards should have only one of these results but we can't
    prevent QEs from making a mistake by moving the card from one lane
    to another, e.g., so just use the last reported result.

    :param trello_db: pymongo.MongoClient db for Trello items
    :param card_id: Trello card id string
    :raises ValueError: if card result not found, e.g. still under
        testing

    """
    result_action = trello_db.actions.find_one(
        {'data.card.id': card_id,
         'data.listAfter.name': {'$in': ['Passed', 'Failed']}},
        sort=[('date', pymongo.DESCENDING)])
    if not result_action:
        raise ValueError(
            "Result action for card {} not found".format(card_id))
    return result_action


def get_card_under_testing_action(trello_db, card_id):
    """Return the 'action' of moving to "Under Testing" column.

    'Actions' come from the Trello API where they're represented in
    JSON; we store them in our db unmodified.  Actions have types,
    users, data, etc. to describe all the actions possible by a user
    in the Trello UI.  We're most interested in the card-moving
    actions which result in a card being moved from one list ("Ready
    for Testing") to another ("Under Testing").

    Silo cards should have only one of these results but we can't
    prevent QEs from making a mistake by moving the card from one lane
    to another, e.g., so just use the last reported result.

    :param trello_db: pymongo.MongoClient db for Trello items
    :param card_id: Trello card id string
    :raises ValueError: if card 'Under Testing' action not found, e.g. 
        not yet under testing

    """
    under_testing_action = trello_db.actions.find_one(
        {'data.card.id': card_id,
         'data.listAfter.name': "Under Testing"},
        sort=[('date', pymongo.DESCENDING)])
    if not under_testing_action:
        raise ValueError(
            "'Under Testing' action for card {} not found".format(card_id))
    return under_testing_action


def get_card_under_testing_datetime(trello_db, card_id):
    """Return the datetime that the given card came under testing.

    :param trello_db: pymongo.MongoClient db for Trello items
    :param card_id: Trello card id string

    """
    try:
        action = get_card_under_testing_action(trello_db, card_id)
    except ValueError:
        # no result yet
        logger.debug(
            "'Under Testing' action for card {} not found.".format(card_id))
        return None
    return dateutil.parser.parse(action['date'])


def get_card_result(trello_db, card_id):
    """Return passed or failed status string for a silo card.

    :param trello_db: pymongo.MongoClient db for Trello items
    :param card_id: Trello card id string

    """
    try:
        action = get_card_result_action(trello_db, card_id)
    except ValueError:
        # no result yet
        logger.debug(
            "No result found for card {}.".format(card_id))
        return None
    return action['data']['listAfter']['name']


def get_card_result_datetime(trello_db, card_id):
    """Return the datetime that the given card passed or failed.

    :param trello_db: pymongo.MongoClient db for Trello items
    :param card_id: Trello card id string

    """
    try:
        action = get_card_result_action(trello_db, card_id)
    except ValueError:
        # no result yet
        logger.debug(
            "No result found for card {}.".format(card_id))
        return None
    return dateutil.parser.parse(action['date'])


def get_card_week_number(trello_db, card_id):
    """Return the week number for the card's result.

    Cards belong to week for which a passed or failed result was
    reported.

    :param trello_db: pymongo.MongoClient db for Trello items
    :param card_id: Trello card id string

    """
    result_datetime = get_card_result_datetime(trello_db, card_id)
    if not result_datetime:
        # no result yet
        logger.debug(
            "No result found for card {}.".format(card_id))
        return None
    return int(result_datetime.strftime("%W"))


def get_card_year(trello_db, card_id):
    """Return the year for the card's result.

    :param trello_db: pymongo.MongoClient db for Trello items
    :param card_id: Trello card id string

    """
    result_datetime = get_card_result_datetime(trello_db, card_id)
    if not result_datetime:
        # no result yet
        logger.debug(
            "No result found for card {}.".format(card_id))
        return None
    return int(result_datetime.strftime("%Y"))


def get_card_testing_duration(trello_db, card_id):
    """Return a silo's testing duration in seconds.

    :param trello_db: pymongo.MongoClient db for Trello items
    :param card_id: Trello card id string

    """
    under_testing_datetime = get_card_under_testing_datetime(
        trello_db, card_id)
    result_datetime = get_card_result_datetime(trello_db, card_id)
    if not under_testing_datetime or not result_datetime:
        logger.debug("Card {} has no testing duration.".format(card_id))
        return None
    duration = result_datetime - under_testing_datetime
    return duration.total_seconds()


def get_card_total_duration(trello_db, card_id):
    """Get a silo's time from creation to passed or failed in seconds.

    :param trello_db: pymongo.MongoClient db for Trello items
    :param card_id: Trello card id string

    """
    created_datetime = get_card_created_datetime(trello_db, card_id)
    result_datetime = get_card_result_datetime(
        trello_db,
        card_id)
    if not created_datetime or not result_datetime:
        logger.debug("Card {} has no total duration.".format(card_id))
        return None
    total_duration = result_datetime - created_datetime
    return total_duration.total_seconds()


def compute_silo_timing(trello_db, card):
    """Return a silo dict with timings, looking up actions in trello_db.

    :param trello_db: pymongo.MongoClient db for Trello items
    :param card: Trello dict of silo card

    """
    logger.info("Computing silo timings for card {}.".format(card['id']))
    silo = {'id': card['id'],
            'name': card['name'],
            'url': card['url']}
    silo['created_datetime'] = get_card_created_datetime(trello_db, card['id'])
    silo['result_datetime'] = get_card_result_datetime(trello_db, card['id'])
    silo['week_number'] = get_card_week_number(trello_db, card['id'])
    silo['year'] = get_card_year(trello_db, card['id'])
    silo['total_duration'] = get_card_total_duration(
        trello_db, card['id'])
    silo['testing_duration'] = get_card_testing_duration(
        trello_db, card['id'])
    silo['result'] = get_card_result(trello_db, card['id'])
    return silo


def compute_silos_timing(trello_db, cards=None):
    """Compute silo timings for the given cards and store in a db.

    :param trello_db: pymongo.MongoClient db in which to store silos
    :param cards: list of card dicts as returned by Trello

    """
    if cards is None:
        cards = trello_db.cards.find()
    for card in cards:
        silo = compute_silo_timing(trello_db, card)
        upsert_silo(trello_db, silo)


def ensure_unique_indexes(trello_db):
    """Ensure that our card, action, and silo indexes exist.

    We're being paranoid--this should've been taken care of in setup
    and we're using upserts everywhere--but we don't want to insert
    duplicates.  NOTE: if this errors it means a duplicate exists,
    needs to be fixed in the db.

    :param trello_db: pymongo.MongoClient db for Trello items

    """
    trello_db.cards.create_index('id', unique=True)
    trello_db.actions.create_index('id', unique=True)
    trello_db.silos.create_index('id', unique=True)


def _read_config(config_filepath):
    """Parse the config at the given filepath, returning a config dict."""
    config_file = configparser.ConfigParser()
    config_file.read(config_filepath)
    config = {}
    for var_name in ('TRELLO_APP_KEY', 'TRELLO_TOKEN', 'TRELLO_BOARD_ID'):
        new_var_name = var_name.lower().replace('trello_', '')
        config[new_var_name] = config_file['DEFAULT'][var_name]
    return config


def _parse_arguments():
    parser = argparse.ArgumentParser(
        "Retrieve silos from Ops Trello board, storing in a Mongo db.")
    parser.add_argument(
        '--since',
        required=False,
        help=("retrieve actions and cards modified since the given date; "
              "defaults to last known action in database."),
        default=None,
        type=str)
    return parser.parse_args()


def _validate_since_arg(args):
    """Validate format of 'since' arg or set to 2014 as necessary.

    :param args: argparse namespace

    """
    if args.since is not None:
        try:
            args.since = datetime.datetime.strptime(args.since, "%Y-%m-%d")
        except ValueError:
            raise argparse.ArgumentError(
                None,
                "'since' must be in YYYY-MM-DD format")


def main():
    util.setup_logging()
    config_dict = _read_config(qakit_config.get_config_file_location())
    args = _parse_arguments()
    _validate_since_arg(args)
    conn = pymongo.MongoClient()
    trello_db = conn.metrics
    ensure_unique_indexes(trello_db)
    trello_session = trelloapi.TrelloApi(
        app_key=config_dict['app_key'],
        token=config_dict['token'])
    cards = trello.retrieve_cards_and_actions(
        trello_db,
        trello_session,
        config_dict['board_id'],
        since=args.since)
    compute_silos_timing(trello_db, cards)


if __name__ == '__main__':
    sys.exit(main())
