# UESQA Metrics
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import logging

import qakit.practitest.practitest as practitest

logger = logging.getLogger(__name__)


def append_testset_id(instance):
    """Append TestSet ID to the given PractiTest instance dict.

    This makes querying much easier later, else it has to be parsed
    out of a string.

    :param instance: a PractiTest instance

    """
    instance['testset_id'] = int(instance['id'].split(':')[0])
    return instance


def append_data_to_instance_dict(instance):
    """Append TestSet ID and last run datetime to the given instance.

    :param instance: a PractiTest instance to which to append data

    """
    instance = practitest.append_last_run_datetime(instance)
    instance = append_testset_id(instance)
    return instance


def update_testset_instances(db, session, testset_id):
    """Retrieve instances from PractiTest for the given TestSet ID.

    NOTE that old runs get crushed on the PractiTest side: e.g. if a
    test fails and it's re-run, the failure is lost to the API.

    :param db: MongoClient db in which to store PractiTest results
    :param practitest_session: PractitestSession
    :param testset_id: the PractiTest TestSet for which to retrieve instances.

    """
    logger.info("Retrieving instances for TestSet {}.".format(testset_id))
    try:
        instances = session.get_instances(testset_id)
    except ValueError as e:
        logger.warn('TestSet not found for id: {}: '.format(
            testset_id), str(e))
        return 0
    count = 0
    for instance in instances:
        instance = append_data_to_instance_dict(instance)
        result = db.instances.update({'system_id': instance['system_id']},
                                     instance,
                                     upsert=True)
        count += result['n']
    logger.info("Retrieved {} instances for TestSet {}.".format(count,
                                                                testset_id))
    return count


def update_testsets_instances(db, practitest_session, testset_ids=None):
    """Update instances for the given testsets, finding new instances.

    An instance represents a test execution in PractiTest.  Note that if a
    test is executed more than once, only the last execution is reported via
    the API: e.g. if a test fails and then is run again and passes, the
    failure is lost to us.

    :param db: a MongoClient db in which to store PractiTest instances
    :param practitest_session: a PractitestSession
    :param testset_ids: list of PractiTest TestSet ids for which to get
        instances: numbers appear in header of TestSet webpage.  If no ids are
        given, retrieve instances for all *known* TestSets (i.e. all already
        in the given database).

    """
    logger.info("Updating TestSets with new instance runs.")
    if testset_ids is None:
        testset_ids = [testset['id'] for testset in db.testsets.find()]
    count = 0
    for testset_id in testset_ids:
        count += update_testset_instances(
            db, practitest_session, testset_id)
    logger.info("Updated {} instances.".format(count))
