# UESQA Metrics
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
import pymongo
from qakit.practitest.practitest import append_last_run_datetime

logger = logging.getLogger(__name__)


def retrieve_testset(db, practitest_session, testset_id):
    """Retrieve a testset of the given ID, storing it in our database.

    :param db: a MongoClient collection in which testsets are stored
    :param practitest_session: a PractiTestSession
    :param testset_id: the ID of the PractiTest testset to retrieve
    :raises ValueError: if testset_id is not found in PractiTest

    """
    try:
        testset = practitest_session.get_testset(testset_id)
    except ValueError:
        logger.warn("Testset {} not found!".format(testset_id))
        raise
    testset = append_last_run_datetime(testset)
    result = db.testsets.update(
        {'system_id': testset['system_id']},
        testset,
        upsert=True)
    if result['updatedExisting']:
        logger.info("Updated testset {}.".format(testset_id))
    else:
        logger.info("Inserted testset {}.".format(testset_id))
    return result


def _get_last_known_testset_id(db):
    """Get the last known testset ID from a db.

    :param db: a MongoClient collection in which testsets are stored

    """
    testsets = db.testsets.find()
    try:
        return testsets.sort('id', pymongo.DESCENDING).limit(1)[0]['id']
    except IndexError:
        raise ValueError("No testsets found in db.")


def update_testsets(db, practitest_session, testset_ids=None):
    """Update PractiTest testsets with new data.

    Note that we update testsets themselves only--instances (i.e. test
    executions) should be updated separately.

    :param db: a MongoClient collection in which testsets are stored
    :param practitest_session: a PractiTestSession
    :param testset_ids: a list of testsets to retrieve; if not specified,
        all known testsets in the given db are updated

    """
    logger.info("Updating existing testsets.")
    if testset_ids is None:
        testset_ids = [testset['id'] for testset in db.testsets.find()]
    count = 0
    for testset_id in testset_ids:
        try:
            result = retrieve_testset(db, practitest_session, testset_id)
            count += result['n']
        except ValueError:
            logger.warn(
                "Testset {} not found, assume deleted.".format(testset_id))
    logger.info("Updated {} testsets.".format(count))


def _scan_for_testsets(db, practitest_session, testset_id_range):
    """Scan a range of testset IDs, retrieving each if it exists in PractiTest.

    Use this to scout ahead while discovering new testsets.

    :param db: a MongoClient collection in which testsets are stored
    :param practitest_session: a PractiTestSession
    :param testset_id_range: a range of testset IDs

    """
    count = 0
    for testset_id in testset_id_range:
        try:
            result = retrieve_testset(db, practitest_session, testset_id)
            count += result['n']
        except ValueError:
            logger.debug("Testset {} not found.".format(testset_id))
    return count


def discover_new_testsets(db, practitest_session, increment=50):
    """Discover and retrieve new testsets in PractiTest.

    PractiTest API won't give a list of IDs of testsets belonging to a
    project.  IDs are assigned incrementally and occasionally deleted;
    scan a given number of IDs ahead and attempt to find a testset at
    each ID, continue until a full scan comes up empty.

    :param db: a MongoClient collection in which testsets are stored
    :param practitest_session: a PractiTestSession
    :param increment: the increment by which to scan ahead, defaults to 50

    """
    logger.info("Discovering new testsets in PractiTest.")
    try:
        last_testset_id = _get_last_known_testset_id(db)
        logger.info("Last known testset is {}.".format(last_testset_id))
    except ValueError as e:
        logger.info("No testsets in db.")
        last_testset_id = 0
    total_count = 0
    testsets_found = True
    next_testset_id = last_testset_id + 1
    while testsets_found:
        testset_id_range = range(next_testset_id, next_testset_id+increment)
        scan_count = _scan_for_testsets(
            db,
            practitest_session,
            testset_id_range)
        if scan_count is 0:
            testsets_found = False
        else:
            next_testset_id = testset_id_range[-1] + 1
        total_count += scan_count
    logger.info("Retrieved {} testsets.".format(total_count))
    return total_count
