# UESQA Metrics
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import datetime
import json
import os

import jinja2


def parse_reporting_arguments(parser):
    """Parse standard input and output reporting arguments.

    :param parser: an argparse parser

    """
    parser.add_argument(
        '-o',
        nargs='*',
        help="Output filepath; .html or .json extension determines format.",
        required=True,
        type=str,
        dest='output_filepaths')
    parser.add_argument(
        '-t',
        help="Path to Jinja2 template for rendering HTML report.",
        required=False,
        type=str,
        dest='html_template_filepath')
    args = parser.parse_args()
    for filepath in args.output_filepaths:
        extension = os.path.splitext(filepath)[-1]
        if extension not in ['.json', '.html']:
            parser.error('Output filepath must end in .json or .html')
        if extension == '.html' and getattr(
                args, 'html_template_filepath') is None:
            parser.error('Must specify template for HTML output.')
    return args


def render_html(
        html_template_filepath, output_filepath, report):
    env = jinja2.environment.Environment()
    env.loader = jinja2.FileSystemLoader(
        os.path.dirname(html_template_filepath))
    template = env.get_template(os.path.basename(html_template_filepath))
    html_report = template.render(report=report)
    html_file = open(output_filepath, 'w')
    html_file.write(html_report)
    html_file.close()


def render_json(output_filepath, report):
    f = open(output_filepath, 'w')
    f.write(json.dumps(report, indent=4))
    f.close()


def render_reports(report, output_filepaths, html_template_filepath=None):
    """Render reporting, types determined by file extension.

    :param report: dict representing report
    :output_filepaths: filepaths for reporting, either .html or .json
    :html_template_filepath: HTML template to render (if .html filepath)

    """
    for output_filepath in output_filepaths:
        extension = os.path.splitext(output_filepath)[-1]
        if extension == '.html':
            if not html_template_filepath:
                raise ValueError("Must specify an HTML template to render!")
            render_html(
                html_template_filepath,
                output_filepath,
                report)
        elif extension == '.json':
            render_json(output_filepath, report)


def get_datetime_from_week_number(week_number, year=2015):
    """Return datetime of end of week of given number."""
    # borrowed from http://stackoverflow.com/questions/17087314/get-date-from-week-number  # NOQA
    return datetime.datetime.strptime(
        "{}-{}-1".format(year, week_number), "%Y-%W-%w")


def get_start_and_end_datetimes_for_week_number(week_number):
    end = get_datetime_from_week_number(week_number)
    start = end - datetime.timedelta(days=7)
    return start, end


def get_last_iso_week_start_datetime(datetime_):
    """Return ISO week start of the given datetime as Python datetime.

    The datetime of midnight (00:00:00) on the Monday before.

    """
    return datetime_ - datetime.timedelta(
        days=(datetime_.isoweekday()-1),
        hours=datetime_.hour,
        minutes=datetime_.minute,
        seconds=datetime_.second,
        microseconds=datetime_.microsecond)


def get_next_iso_week_start_datetime(datetime_):
    """Return ISO week start of the week following the given datetime.

    The datetime of midnight (00:00:00) on the Monday after.

    """
    return (get_last_iso_week_start_datetime(datetime_) +
            datetime.timedelta(days=7))


def compose_week_series(start, end):
    last_week_start = start
    next_week_start = get_next_iso_week_start_datetime(start)
    periods = []
    while next_week_start < end:
        periods.append((last_week_start, next_week_start))
        last_week_start = next_week_start
        next_week_start = last_week_start + datetime.timedelta(days=7)
    periods.append((last_week_start, end))
    return periods
