#!/usr/bin/python3
# UESQA Metrics
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import argparse
import datetime
import logging
import sys

import pymongo

import qakit.metrics.reports as reports
import qakit.metrics.util as util

logger = logging.getLogger(__name__)


def get_total_count(db, start, end):
    return db.silos.find(
        {'result_datetime': {'$gt': start, '$lt': end}}).count()


def get_passed_count(db, start, end):
    return db.silos.find(
        {'result_datetime': {'$gt': start, '$lt': end},
         'result': 'Passed'}).count()


def get_failed_count(db, start, end):
    return db.silos.find(
        {'result_datetime': {'$gt': start, '$lt': end},
         'result': 'Failed'}).count()


def get_most_expensive_silo(db, start, end):
    max_silo = [
        {'$match': {'result_datetime': {'$gt': start, '$lt': end}}},
        {'$group': {'_id': None, 'result': {'$max': '$testing_duration'}}}
    ]
    try:
        result = db.silos.aggregate(max_silo)['result'][0]['result']
    except IndexError:
        # no silos
        return None
    try:
        return result/60/60
    except TypeError:
        # weird but possible, testing_duration was zero
        return 0


def get_cheapest_silo(db, start, end):
    min_silo = [
        {'$match': {'result_datetime': {'$gt': start, '$lt': end}}},
        {'$group': {'_id': None, 'result': {'$min': '$testing_duration'}}}
    ]
    try:
        result = db.silos.aggregate(min_silo)['result'][0]['result']
    except IndexError:
        # no silos
        return None
    try:
        return result/60/60
    except TypeError:
        # weird but possible, testing_duration was zero
        return 0


def get_average_silo(db, start, end):
    avg_silo = [
        {'$match': {'result_datetime': {'$gt': start, '$lt': end}}},
        {'$group': {'_id': None, 'result': {'$avg': '$testing_duration'}}}
    ]
    try:
        result = db.silos.aggregate(avg_silo)['result'][0]['result']
    except IndexError:
        # no silos
        return None
    try:
        return result/60/60
    except TypeError:
        # weird but possible, average testing_duration was zero
        return 0


def get_time_spent_on_failures(db, start, end):
    total_failed_silos = [
        {'$match': {'result_datetime': {'$gt': start, '$lt': end},
                    'result': 'Failed'}},
        {'$group': {'_id': None, 'result': {'$sum': '$testing_duration'}}}]
    try:
        result = db.silos.aggregate(total_failed_silos)['result'][0]['result']
    except IndexError:
        # no failures
        return None
    try:
        return result/60/60
    except TypeError:
        # we spent no time on failures
        return 0


def get_landings_counts(db, start, end):
    """Generate test library JSON for the given week number.

    Report on total number of tests, automated test counts, etc.

    """
    result = {}
    result['total'] = get_total_count(db, start, end)
    result['passed'] = get_passed_count(db, start, end)
    result['failed'] = get_failed_count(db, start, end)
    result['average_time'] = get_average_silo(db, start, end)
    result['time_spent_on_failures'] = get_time_spent_on_failures(
        db, start, end)
    result['cheapest'] = get_cheapest_silo(db, start, end)
    result['most_expensive'] = get_most_expensive_silo(db, start, end)
    try:
        result['pass_rate'] = result['passed'] / result['total'] * 100
    except ZeroDivisionError:
        result['pass_rate'] = 0
    return result


def get_weekly_report(db, start, end):
    report = get_landings_counts(db, start, end)
    report['week_number'] = start.isocalendar()[1]
    report['start'] = start.strftime('%Y-%m-%d')
    report['end'] = start.strftime('%Y-%m-%d')
    return report


def get_landings_report(db, start=None, end=None):
    if start is None:
        # we have instance data back to mid-May
        start = datetime.datetime(2015, 1, 1)
    if end is None:
        end = datetime.datetime.now()
    weeks = reports.compose_week_series(start, end)
    weeks.reverse()
    weekly_reports = []
    for start, end in weeks:
        try:
            weekly_report = get_weekly_report(db, start, end)
        except ValueError as e:
            logger.warn("{}: {}".format(start.isoformat(), e))
            continue
        weekly_reports.append(weekly_report)
    return weekly_reports


def _parse_arguments():
    parser = argparse.ArgumentParser("Report landings from Trello.")
    return reports.parse_reporting_arguments(parser)


def main():
    util.setup_logging()
    args = _parse_arguments()
    conn = pymongo.MongoClient()
    db = conn.metrics
    report = get_landings_report(db)
    reports.render_reports(
        report, args.output_filepaths, args.html_template_filepath)


if __name__ == '__main__':
    sys.exit(main())
