#!/usr/bin/python3
# UESQA Metrics
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import argparse
import datetime
import logging
import os
import sys

import pymongo

import qakit.metrics.reports as reports
import qakit.metrics.util as util

logger = logging.getLogger(__name__)


def get_total_count(db, start, end):
    return db.instances.find(
        {'last_run_datetime': {'$gt': start, '$lt': end}}).count()


def get_manual_count(db, start, end):
    return db.instances.find(
        {'last_run_datetime': {'$gt': start, '$lt': end},
         'tester.value': {'$ne': None}}).count()


def get_automated_count(db, start, end):
    return db.instances.find(
        {'last_run_datetime': {'$gt': start, '$lt': end},
         'tester.value': None}).count()


def get_tester_count(db, tester_name, start, end):
    """Return the test count for a given tester for a given week."""
    return db.instances.find(
        {'last_run_datetime': {'$gt': start, '$lt': end},
         'tester.value': tester_name}).count()


def get_tester_list_count(db, tester_name_list, start, end):
    """Return the test count for a given tester for a given week."""
    return db.instances.find(
        {'last_run_datetime': {'$gt': start, '$lt': end},
         'tester.value': {'$in': tester_name_list}}).count()


def get_distinct_manual_count(db, start, end):
    distinct_test_count = db.instances.aggregate(
        [{'$match': {'last_run_datetime': {'$gt': start, '$lt': end},
                     'tester.value': {'$ne': None}}},
         {'$group': {'_id': '$name'}},
         {'$group': {'_id': None, 'count': {'$sum': 1}}}])
    try:
        return distinct_test_count['result'][0]['count']
    except IndexError:
        # no "result"
        return 0


def get_distinct_automated_count(db, start, end):
    distinct_test_count = db.instances.aggregate(
        [{'$match': {'last_run_datetime': {'$gt': start, '$lt': end},
                     'tester.value': None}},
         {'$group': {'_id': '$name'}},
         {'$group': {'_id': None, 'count': {'$sum': 1}}}])
    try:
        return distinct_test_count['result'][0]['count']
    except IndexError:
        # no "result"
        return 0


def get_manual_run_status_count(db, start, end, run_status):
    return db.instances.find(
        {'last_run_datetime': {'$gt': start, '$lt': end},
         'run_status.value': run_status,
         'tester.value': {'$ne': None}}).count()


def get_automated_run_status_count(db, start, end, run_status):
    return db.instances.find(
        {'last_run_datetime': {'$gt': start, '$lt': end},
         'run_status.value': run_status,
         'tester.value': None}).count()


def get_test_execution_counts(db, directory_db, start, end):
    """Query for results and print out a report."""
    result = {}
    result['total'] = get_total_count(db, start, end)
    result['manual'] = get_manual_count(db, start, end)
    result['automated'] = get_automated_count(db, start, end)
    result['distinct_manual'] = get_distinct_manual_count(db, start, end)
    result['distinct_automated'] = get_distinct_automated_count(db, start, end)
    result['manual_passed'] = get_manual_run_status_count(
        db, start, end, 'PASSED')
    result['manual_failed'] = get_manual_run_status_count(
        db, start, end, 'FAILED')
    result['manual_blocked'] = get_manual_run_status_count(
        db, start, end, 'BLOCKED')
    result['manual_not_completed'] = get_manual_run_status_count(
        db, start, end, 'NOT COMPLETED')
    result['automated_passed'] = get_automated_run_status_count(
        db, start, end, 'PASSED')
    result['automated_failed'] = get_automated_run_status_count(
        db, start, end, 'FAILED')
    return result


def get_weekly_report(db, directory_db, start, end):
    report = get_test_execution_counts(db, directory_db, start, end)
    report['week_number'] = start.isocalendar()[1]
    report['start'] = start.strftime('%Y-%m-%d')
    report['end'] = start.strftime('%Y-%m-%d')
    return report


def get_test_execution_report(db, directory_db, start=None, end=None):
    if start is None:
        # we have instance data back to mid-May
        start = datetime.datetime(2015, 5, 18)
    if end is None:
        end = datetime.datetime.now()
    weeks = reports.compose_week_series(start, end)
    weeks.reverse()
    weekly_reports = []
    for start, end in weeks:
        try:
            weekly_report = get_weekly_report(db, directory_db, start, end)
        except ValueError as e:
            logger.warn("{}: {}".format(start.isoformat(), e))
            continue
        weekly_reports.append(weekly_report)
    return weekly_reports


def _parse_arguments():
    parser = argparse.ArgumentParser("Report PractiTest test executions.")
    return reports.parse_reporting_arguments(parser)


def main():
    util.setup_logging()
    args = _parse_arguments()
    conn = pymongo.MongoClient()
    db = conn.metrics
    directory_db = conn.directory
    report = get_test_execution_report(db, directory_db)
    reports.render_reports(
        report, args.output_filepaths, args.html_template_filepath)


if __name__ == '__main__':
    sys.exit(main())
