#!/usr/bin/python3
# UESQA Metrics
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import argparse
import datetime
import logging
import re
import sys

import pymongo

import qakit.practitest.practitest as practitest
import qakit.metrics.reports as reports
import qakit.metrics.util as util

logger = logging.getLogger(__name__)


def get_last_run_datetimes(db, start, end):
    """Return the last datetime for which a full scan occurred.

    :param since: the datetime before which to find a complete scan

    """
    try:
        last_run = db.test_history_runs.find(
            {'end': {'$lt': end},
             'start': {'$gt': start}}).sort(
                 'end', pymongo.DESCENDING).limit(1)[0]
    except IndexError:
        raise ValueError("No data from {} to {}.".format(
            start, end))
    return last_run['start'], last_run['end']


def get_total_count(db, start, end):
    """Return a dict of 'total' number of tests.

    """
    return db.test_history.find(
        {'datetime': {'$gt': start, '$lt': end}}).count()


def get_status_count(db, start, end, status):
    return db.test_history.find(
        {'datetime': {'$gt': start, '$lt': end},
         'test_status.value': status}).count()


def get_other_status_count(db, start, end):
    return db.test_history.find(
        {'datetime': {'$gt': start, '$lt': end},
         '$and': [{'test_status.value': {'$ne': "To Review"}},
                  {'test_status.value': {'$ne': "Ready"}}]}).count()


def get_automated_count(db, start, end):
    """Return the number of automated tests in our database.

    NOTE that we only consider tests which are "Ready".

    """
    return db.test_history.find(
        {'datetime': {'$gt': start, '$lt': end},
         "{}.value".format(practitest.EXTERNAL_ID): re.compile(
             "^ubuntu_system_tests"),
         'test_status.value': "Ready"}).count()


def get_manual_count(db, start, end):
    """Return the number of manual tests in our database.

    NOTE that we only consider tests which are "Ready".

    """
    return db.test_history.find(
        {'datetime': {'$gt': start, '$lt': end},
         "{}.value".format(practitest.EXTERNAL_ID): None,
         'test_status.value': "Ready"}).count()


def get_test_library_counts(db, week_start, week_end):
    """Get test library JSON for the given week number.

    Report on total number of tests, automated test counts, etc.

    """
    result = {}
    start, end = get_last_run_datetimes(db, week_start, week_end)
    result['total'] = get_total_count(db, start, end)
    result['ready'] = get_status_count(db, start, end, 'Ready')
    result['to_review'] = get_status_count(db, start, end, 'To Review')
    result['other_status'] = get_other_status_count(db, start, end)
    result['manual_ready'] = get_manual_count(db, start, end)
    result['automated_ready'] = get_automated_count(db, start, end)
    try:
        result['percent_automated'] = (result['automated_ready'] /
                                       result['ready'])
    except ZeroDivisionError:
        result['percent_automated'] = 0
    return result


def get_weekly_report(db, start, end):
    report = get_test_library_counts(db, start, end)
    report['week_number'] = start.isocalendar()[1]
    report['start'] = start.strftime('%Y-%m-%d')
    report['end'] = start.strftime('%Y-%m-%d')
    return report


def get_test_library_report(db, start=None, end=None):
    if start is None:
        # we have library ata back to mid-May
        start = datetime.datetime(2015, 5, 18)
    if end is None:
        end = datetime.datetime.now()
    weeks = reports.compose_week_series(start, end)
    weeks.reverse()
    weekly_reports = []
    for start, end in weeks:
        try:
            weekly_report = get_weekly_report(db, start, end)
        except ValueError as e:
            logger.warn("{}: {}".format(start.isoformat(), e))
            continue
        weekly_reports.append(weekly_report)
    return weekly_reports


def _parse_arguments():
    parser = argparse.ArgumentParser(
        "Report PractiTest test library activity.")
    return reports.parse_reporting_arguments(parser)


def main():
    util.setup_logging()
    args = _parse_arguments()
    conn = pymongo.MongoClient()
    db = conn.metrics
    report = get_test_library_report(db, end=datetime.datetime.now())
    reports.render_reports(
        report, args.output_filepaths, args.html_template_filepath)


if __name__ == '__main__':
    sys.exit(main())
