#!/usr/bin/python3
# UESQA Metrics
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import argparse
import logging
import sys

import pymongo

from qakit import config
from qakit.metrics.practitest import instances
from qakit.metrics.practitest import testsets
from qakit.metrics import util
import qakit.practitest.practitest as practitest

logger = logging.getLogger(__name__)


def update_testsets(db, practitest_session, testset_ids=None):
    """Discover new and update existing PractiTest testsets.

    :param db: a MongoClient collection in which PractiTest data is stored
    :param practitest_session: a PractiTestSession
    :param testsets: a list of PractiTest testset ids to update; if not
        specified, new testsets are discovered and all existing testsets are
        updated

    """
    if not testset_ids:
        testset_ids = [testset['id'] for testset in db.testsets.find()]
        # (newly-discovered testsets will be updated upon discovery)
        testsets.discover_new_testsets(db, practitest_session)
    testsets.update_testsets(db, practitest_session, testset_ids=testset_ids)


def update_instances(db, practitest_session, testset_ids=None):
    """Discover new and update existing instances for the given testset IDs.

    :param db: a MongoClient collection in which PractiTest data is stored
    :param practitest_session: a PractiTestSession
    :param testsets: a list of PractiTest testset ids to update; if not
        specified, all known testsets are updated

    """
    if not testset_ids:
        testset_ids = [testset['id'] for testset in db.testsets.find()]
    instances.update_testsets_instances(
        db,
        practitest_session,
        testset_ids=testset_ids)


def update_test_execution_data(db, practitest_session, testset_ids=None):
    """Update PractiTest test execution data: testsets and instances.

    :param db: a MongoClient collection in which PractiTest data is stored
    :param practitest_session: a PractiTestSession
    :param testsets: a list of PractiTest testset ids to update; if not
        specified, new testsets are discovered and all existing testsets are
        updated

    """
    update_testsets(db, practitest_session, testset_ids=testset_ids)
    update_instances(db, practitest_session, testset_ids=testset_ids)


def _parse_arguments():
    parser = argparse.ArgumentParser(
        "Retrieve PractiTest results, storing them in a MongoDB database.")
    parser.add_argument(
        nargs='*',
        dest='testsets',
        help='TestSet from which to retrieve instances',
        type=int)
    return parser.parse_args()


def main():
    util.setup_logging()
    config_dict = util.read_config(
        config.get_config_file_location())
    args = _parse_arguments()
    conn = pymongo.MongoClient()
    db = conn.metrics
    practitest_session = practitest.PractitestSession(
        config_dict['project_id'],
        config_dict['api_key'],
        config_dict['api_secret_key'])
    update_test_execution_data(
        db,
        practitest_session,
        testset_ids=args.testsets)


if __name__ == '__main__':
    try:
        sys.exit(main())
    except Exception as e:
        logger.error(e, exc_info=True)
