# UESQA Metrics
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import datetime
import unittest
from unittest import mock

import dateutil
from bson import ObjectId

import qakit.metrics.landings as landings


CREATED = {
    "_id": ObjectId("559d3b9a6f9289b4a62909fd"),
    "id": "559c033efbfb3ecbc9f725b6",
    "type": "createCard",
    "memberCreator": {
        "id": "53bfc3936a7d7c888e46f4cd",
        "fullName": "Jean-Baptiste Lallement",
        "username": "jeanbaptistelallement",
        "initials": "JBL",
        "avatarHash": None
    },
    "date": "2015-07-07T16:50:06.409Z",
    "data": {
        "board": {
            "id": "53fc6641728df958a48bfbe1",
            "shortLink": "AE3swczu",
            "name": ("QA Testing Requests - for questions ping EU: "
                     "jibel, US: jfunk - or ubuntu-qa on #ubuntu-ci-eng")
        },
        "card": {
            "name": "ubuntu/landing-026 - messaging-app : boiko",
            "id": "559c033efbfb3ecbc9f725b5",
            "shortLink": "7cOV5cYP",
            "idShort": 1865
        },
        "list": {
            "id": "53fc6641728df958a48bfbe2",
            "name": "Need QA Sign-off"
        }
    },
    "idMemberCreator": "53bfc3936a7d7c888e46f4cd"
}

UNDER_TESTING = {
    "_id": ObjectId("559d3b9a6f9289b4a62909f4"),
    "id": "559c50bfb233bddda9939772",
    "type": "updateCard",
    "memberCreator": {
        "id": "510c44e279f3442c56000417",
        "fullName": "Allan LeSage",
        "username": "allanlesage",
        "initials": "AL",
        "avatarHash": "5c61e74f355449f6c7cfc35880e6a49f"
    },
    "date": "2015-07-07T22:20:47.469Z",
    "data": {
        "board": {
            "id": "53fc6641728df958a48bfbe1",
            "shortLink": "AE3swczu",
            "name": ("QA Testing Requests - for questions ping EU: "
                     "jibel, US: jfunk - or ubuntu-qa on #ubuntu-ci-eng")
        },
        "card": {
            "name": "ubuntu/landing-026 - messaging-app : boiko",
            "id": "559c033efbfb3ecbc9f725b5",
            "shortLink": "7cOV5cYP",
            "idList": "53fc6641728df958a48bfbe3",
            "idShort": 1865
        },
        "listBefore": {
            "id": "552bcb7e2bc13035850abeae",
            "name": "Ready for Testing"
        },
        "listAfter": {
            "id": "53fc6641728df958a48bfbe3",
            "name": "Under Testing"
        },
        "old": {
            "idList": "552bcb7e2bc13035850abeae"
        }
    },
    "idMemberCreator": "510c44e279f3442c56000417"
}

PASSED = {
    "_id": ObjectId("559d3b9a6f9289b4a62909ed"),
    "id": "559d24feed5efa44c31650c0",
    "type": "updateCard",
    "memberCreator": {
        "id": "53bfc49275645e39d37a1046",
        "fullName": "Víctor R. Ruiz",
        "username": "victorrruiz",
        "initials": "VR",
        "avatarHash": "c15c5a5db02d89903f2233d88ad778d5"
    },
    "date": "2015-07-08T13:26:22.006Z",
    "data": {
        "board": {
            "id": "53fc6641728df958a48bfbe1",
            "shortLink": "AE3swczu",
            "name": ("QA Testing Requests - for questions ping EU: "
                     "jibel, US: jfunk - or ubuntu-qa on #ubuntu-ci-eng")
        },
        "card": {
            "name": "ubuntu/landing-026 - messaging-app : boiko",
            "id": "559c033efbfb3ecbc9f725b5",
            "shortLink": "7cOV5cYP",
            "idList": "53fc6641728df958a48bfbe4",
            "idShort": 1865
        },
        "listBefore": {
            "id": "53fc6641728df958a48bfbe3",
            "name": "Under Testing"
        },
        "listAfter": {
            "id": "53fc6641728df958a48bfbe4",
            "name": "Passed"
        },
        "old": {
            "idList": "53fc6641728df958a48bfbe3"
        }
    },
    "idMemberCreator": "53bfc49275645e39d37a1046"
}


class GetCardCreatedActionTestCase(unittest.TestCase):

    def test_vanilla(self):
        db = mock.Mock(
            actions=mock.Mock(
                find_one=mock.Mock(return_value=CREATED)))
        self.assertEqual(
            CREATED,
            landings.get_card_created_action(db, 'fake-card-id'))

    def test_no_created_action_raises(self):
        db = mock.Mock(
            actions=mock.Mock(
                find_one=mock.Mock(return_value=None)))
        with self.assertRaises(ValueError):
            landings.get_card_created_action(db, 'fake-card-id')


@mock.patch('qakit.metrics.landings.get_card_created_action')
class GetCardCreatedDatetimeTestCase(unittest.TestCase):

    def test_vanilla(self, mock_get_card_created_action):
        mock_get_card_created_action.return_value = CREATED
        self.assertEqual(
            datetime.datetime(
                2015, 7, 7, 16, 50, 6, 409000, tzinfo=dateutil.tz.tzutc()),
            landings.get_card_created_datetime('fake-db', 'fake-card-id'))

    def test_no_created_datetime_raises(self, mock_get_card_created_action):
        mock_get_card_created_action.side_effect = ValueError
        self.assertEqual(
            None,
            landings.get_card_created_datetime('fake-db', 'fake-card-id'))


class GetCardResultActionTestCase(unittest.TestCase):

    def test_vanilla(self):
        db = mock.Mock(
            actions=mock.Mock(
                find_one=mock.Mock(return_value=PASSED)))
        action = landings.get_card_result_action(db, 'fake-card-id')
        self.assertEqual(PASSED, action)

    def test_raise_if_action_not_found(self):
        db = mock.Mock(
            actions=mock.Mock(
                find_one=mock.Mock(return_value=None)))
        with self.assertRaises(ValueError):
            landings.get_card_result_action(db, 'fake-card-id')


@mock.patch('qakit.metrics.landings.get_card_result_action')
class GetCardResultDatetimeTestCase(unittest.TestCase):

    def test_vanilla(self, mock_get_card_result_action):
        mock_get_card_result_action.return_value = PASSED
        self.assertEqual(
            datetime.datetime(
                2015, 7, 8, 13, 26, 22, 6000, tzinfo=dateutil.tz.tzutc()),
            landings.get_card_result_datetime('fake-db', 'fake-card-id'))

    def test_no_result_datetime_if_no_result(self, mock_get_card_result_action):
        mock_get_card_result_action.side_effect = ValueError
        self.assertEqual(
            None,
            landings.get_card_result_datetime('fake-db', 'fake-card-id'))


class GetCardUnderTestingActionTestCase(unittest.TestCase):

    def test_vanilla(self):
        db = mock.Mock(
            actions=mock.Mock(
                find_one=mock.Mock(return_value=UNDER_TESTING)))
        action = landings.get_card_under_testing_action(db, 'fake-card-id')
        self.assertEqual(UNDER_TESTING, action)

    def test_raise_if_action_not_found(self):
        db = mock.Mock(
            actions=mock.Mock(
                find_one=mock.Mock(return_value=None)))
        with self.assertRaises(ValueError):
            landings.get_card_under_testing_action(db, 'fake-card-id')


@mock.patch('qakit.metrics.landings.get_card_under_testing_action')
class GetCardUnderTestingDatetimeTestCase(unittest.TestCase):

    def test_vanilla(self, mock_get_card_under_testing_action):
        mock_get_card_under_testing_action.return_value = PASSED
        self.assertEqual(
            datetime.datetime(
                2015, 7, 8, 13, 26, 22, 6000, tzinfo=dateutil.tz.tzutc()),
            landings.get_card_under_testing_datetime('fake-db', 'fake-card-id'))

    def test_no_under_testing_datetime_if_never_under_testing(
            self, mock_get_card_under_testing_action):
        mock_get_card_under_testing_action.side_effect = ValueError
        self.assertEqual(
            None,
            landings.get_card_under_testing_datetime('fake-db', 'fake-card-id'))


@mock.patch('qakit.metrics.landings.get_card_result_action')
class GetCardResultTestCase(unittest.TestCase):

    def test_vanilla(self, mock_get_card_result_action):
        mock_get_card_result_action.return_value = PASSED
        self.assertEqual("Passed", landings.get_card_result(
            'fake-db', 'fake-card-id'))

    def test_no_result_if_no_result(self, mock_get_card_result_action):
        mock_get_card_result_action.side_effect = ValueError
        self.assertEqual(None, landings.get_card_result(
            'fake-db', 'fake-card-id'))


@mock.patch('qakit.metrics.landings.get_card_result_datetime')
class GetCardWeekNumberTestCase(unittest.TestCase):

    def test_vanilla(self, mock_get_card_result_datetime):
        mock_get_card_result_datetime.return_value = datetime.datetime(
            2015, 7, 8, 13, 26, 22, 6000, tzinfo=dateutil.tz.tzutc())
        self.assertEqual(
            27,
            landings.get_card_week_number('fake-db', 'fake-card-id'))

    def test_no_week_number_if_no_result(self, mock_get_card_result_datetime):
        mock_get_card_result_datetime.return_value = None
        self.assertEqual(None, landings.get_card_week_number(
            'fake-db', 'fake-card-id'))


@mock.patch('qakit.metrics.landings.get_card_result_datetime')
class GetCardYearTestCase(unittest.TestCase):

    def test_vanilla(self, mock_get_card_result_datetime):
        mock_get_card_result_datetime.return_value = datetime.datetime(
            2015, 7, 8, 13, 26, 22, 6000, tzinfo=dateutil.tz.tzutc())
        self.assertEqual(
            2015,
            landings.get_card_year('fake-db', 'fake-card-id'))

    def test_no_year_if_no_result(self, mock_get_card_result_datetime):
        mock_get_card_result_datetime.return_value = None
        self.assertEqual(None, landings.get_card_year(
            'fake-db', 'fake-card-id'))


@mock.patch('qakit.metrics.landings.get_card_result_datetime')
@mock.patch('qakit.metrics.landings.get_card_under_testing_datetime')
class GetCardTestingDurationTestCase(unittest.TestCase):

    def test_vanilla(
            self,
            mock_get_card_under_testing_datetime,
            mock_get_card_result_datetime):
        mock_get_card_under_testing_datetime.return_value = datetime.datetime(
            2015, 7, 8, 13, 25, 22, 6000, tzinfo=dateutil.tz.tzutc())
        mock_get_card_result_datetime.return_value = datetime.datetime(
            2015, 7, 8, 13, 26, 22, 6000, tzinfo=dateutil.tz.tzutc())
        self.assertEqual(60.0, landings.get_card_testing_duration(
            'fake-db', 'fake-card-id'))

    def test_no_under_testing_datetime(
            self,
            mock_get_card_under_testing_datetime,
            mock_get_card_result_datetime):
        mock_get_card_under_testing_datetime.return_value = datetime.datetime(
            2015, 7, 8, 13, 25, 22, 6000, tzinfo=dateutil.tz.tzutc())
        mock_get_card_result_datetime.return_value = None
        self.assertEqual(None, landings.get_card_testing_duration(
            'fake-db', 'fake-card-id'))


@mock.patch('qakit.metrics.landings.get_card_result_datetime')
@mock.patch('qakit.metrics.landings.get_card_created_datetime')
class GetCardTotalDurationTestCase(unittest.TestCase):

    def test_vanilla(
            self,
            mock_get_card_created_datetime,
            mock_get_card_result_datetime):
        mock_get_card_created_datetime.return_value = datetime.datetime(
            2015, 7, 8, 13, 25, 22, 6000, tzinfo=dateutil.tz.tzutc())
        mock_get_card_result_datetime.return_value = datetime.datetime(
            2015, 7, 8, 13, 26, 22, 6000, tzinfo=dateutil.tz.tzutc())
        self.assertEqual(60.0, landings.get_card_total_duration(
            'fake-db', 'fake-card-id'))

    def test_no_under_testing_datetime(
            self,
            mock_get_card_created_datetime,
            mock_get_card_result_datetime):
        mock_get_card_created_datetime.return_value = datetime.datetime(
            2015, 7, 8, 13, 25, 22, 6000, tzinfo=dateutil.tz.tzutc())
        mock_get_card_result_datetime.return_value = None
        self.assertEqual(None, landings.get_card_total_duration(
            'fake-db', 'fake-card-id'))
