# UESQA Metrics
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import logging

import pymongo

logger = logging.getLogger(__name__)


def upsert_card(trello_db, card):
    """Insert a Trello card into our db.

    :param trello_db: pymongo.MongoClient db for Trello items
    :param action: Trello-reported action to upsert

    """
    try:
        trello_db.cards.update(
            {'id': card['id']},
            card,
            upsert=True)
    except pymongo.errors.PyMongoError as e:
        logger.error(e)


def upsert_action(trello_db, action):
    """Insert a Trello action into our db.

    :param trello_db: pymongo.MongoClient db for Trello items
    :param action: Trello-reported action to upsert

    """
    try:
        return trello_db.actions.update(
            {'id': action['id']},
            action,
            upsert=True)
    except pymongo.errors.PyMongoError as e:
        logger.error(e)


def upsert_actions(trello_db, actions):
    """Insert a list of Trello silo actions into our trello_db.

    :param trello_db: pymongo.MongoClient db for Trello items
    :param actions: Trello-reported actions to upsert

    """
    # TODO: does mongo make it possible to upsert the array whole?
    for action in actions:
        try:
            upsert_action(trello_db, action)
        except pymongo.errors.PyMongoError as e:
            logger.error(e)


def get_last_action_datetime(trello_db):
    """Return the datetime of the last known action in our db.

    :param trello_db: pymongo.MongoClient db for Trello items

    """
    last_action = trello_db.actions.find_one(
        {},
        sort=[('date', pymongo.DESCENDING)])
    try:
        return last_action['date']
    except TypeError:
        # first run we have no actions, e.g.
        return None


def get_boards_actions_count(trello_session, board_id, since=None):
    """Return number of actions available for given board_id.

    NOTE that 1000 is Trello's max, likely means that there are more.

    :param trello_session: TrelloSession to query
    :param board_id: Trello board id string
    :param since: datetime to query, defaults to None

    """
    return trello_session.get_boards_action(
        board_id,
        limit=1000,
        trello_format='count',
        since=since)['_value']


def get_board_actions_by_card(trello_db,
                              trello_session,
                              board_id,
                              since=None):
    """Crawl a board's cards, retrieving actions and storing in db.

    Trello API limits us to 1K actions when querying by board--use
    this when spooling up a db from scratch, e.g., or when auditing.

    :param trello_db: pymongo.MongoClient db for Trello items
    :param trello_session: TrelloSession to query
    :param board_id: Trello board id string
    :param since: datetime from which to query, defaults to None

    """
    cards = trello_session.get_boards_card(
        board_id,
        filter='all',
        since=since)
    for card in cards:
        actions = trello_session.get_cards_actions(
            card['id'],
            filter=("addMemberToCard,removeMemberFromCard,"
                    "commentCard,updateCard,createCard"),
            since=since)
        logger.debug('Card "{}" has {} actions.'.format(
            card['name'], len(actions)))
        upsert_actions(trello_db, actions)


def retrieve_board_cards(trello_db, trello_session, board_id, since=None):
    """Retrieve cards associated with a board and store in trello_db.

    :param trello_db: pymongo.MongoClient db for Trello items
    :param trello_session: TrelloSession to query
    :param board_id: Trello board id string
    :param since: datetime from which to query, defaults to None

    """
    if not since:
        since = get_last_action_datetime(trello_db)
    cards = trello_session.get_boards_card(
        board_id,
        filter='all',
        since=since)
    logger.info("Board {} contains {} cards.".format(board_id, len(cards)))
    for card in cards:
        upsert_card(trello_db, card)
    return cards


def retrieve_board_actions(trello_db, trello_session, board_id, since=None):
    """Get actions associated with a board from Trello and store in trello_db.

    :param trello_db: pymongo.MongoClient db for Trello items
    :param trello_session: TrelloSession to query
    :param board_id: Trello board id string
    :param since: datetime to query, defaults to None

    """
    if not since:
        since = get_last_action_datetime(trello_db)
        logger.debug("Retr{}.".format(since))
    if get_boards_actions_count(
            trello_session,
            board_id,
            since=since) == 1000:
        logger.info("More than 1000 actions found; crawling cards "
                    "individually to get a complete set of actions.")
        get_board_actions_by_card(trello_db, trello_session, board_id, since)
    else:
        actions = trello_session.get_boards_action(
            board_id,
            since=since)
        logger.info("Found {} actions since {}.".format(
            len(actions), since))
        for action in actions:
            upsert_action(trello_db, action)


def retrieve_cards_and_actions(trello_db,
                               trello_session,
                               board_id,
                               since=None):
    """Get a board's cards and actions, store in trello_db, and return cards.

    :param trello_db: pymongo.MongoClient db for Trello items
    :param trello_session: TrelloSession to query
    :param board_id: Trello board id string
    :param since: datetime to query, defaults to None; if None get all actions
        and cards since the last known action in our db

    """
    if not since:
        since = get_last_action_datetime(trello_db)
        logger.debug("Last action occurred {}.".format(since))
    logger.debug("Retrieving cards and actions for board {} since {}.".format(
        board_id, since))
    cards = retrieve_board_cards(
        trello_db,
        trello_session,
        board_id,
        since=since)
    retrieve_board_actions(trello_db, trello_session, board_id, since=since)
    return cards
