
# UEQA KPIs
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import configparser
import functools
import sys
import time

import logging


LOG_FILEPATH = '/var/log/qakit.log'


def setup_logging(log_filepath=LOG_FILEPATH):
    """Set up qakit logging.

    :param log_filepath: filepath for system log, suggest /var/log/qakit.log

    """
    formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
    file_handler = logging.FileHandler(log_filepath)
    file_handler.setLevel(logging.DEBUG)
    file_handler.setFormatter(formatter)
    stdout_handler = logging.StreamHandler(sys.stdout)
    stdout_handler.setLevel(logging.DEBUG)
    stdout_handler.setFormatter(formatter)
    logger = logging.getLogger()
    logger.setLevel(logging.DEBUG)
    logger.addHandler(stdout_handler)
    logger.addHandler(file_handler)


def retry(ExceptionToCheck, tries=4, delay=2, backoff=2, logger=None):
    """Retry calling the decorated function using an exponential backoff.

    http://www.saltycrane.com/blog/2009/11/trying-out-retry-decorator-python/
    Original from: http://wiki.python.org/moin/PythonDecoratorLibrary#Retry

    :param ExceptionToCheck: the exception to check. may be a tuple of
        exceptions to check
    :param tries: number of times to try (not retry) before giving up
    :param delay: initial delay between retries in seconds
    :param backoff: backoff multiplier e.g. value of 2 will double the delay
        each retry
    :param logger: logger.Logger to use.

    """
    def deco_retry(f):
        @functools.wraps(f)
        def f_retry(*args, **kwargs):
            for i in range(tries):
                try:
                    return f(*args, **kwargs)
                except ExceptionToCheck as e:
                    d = delay * backoff ** i
                    if logger:
                        logger.debug(
                            "%s, retrying in %d seconds. . . ." % (e, d))
                    time.sleep(d)
            return f(*args, **kwargs)
        return f_retry  # true decorator
    return deco_retry


def get_week_number(datetime_):
    """Return the week of the year from a datetime."""
    return int(datetime_.strftime('%W'))


def read_config(config_filepath):
    """Parse the config at the given filepath, returning a config dict.

    :param config_filepath: the filepath to a configuration file

    """
    config_file = configparser.ConfigParser()
    config_file.read(config_filepath)
    config = {}
    for var_name in ('PRACTITEST_PROJECT_ID',
                     'PRACTITEST_API_KEY',
                     'PRACTITEST_API_SECRET_KEY'):
        new_var_name = var_name.lower().replace('practitest_', '')
        config[new_var_name] = config_file['DEFAULT'][var_name]
    return config
