#!/usr/bin/python3

# Sync issues between LP and PT

# Copyright (C) 2015 Canonical
#
# Author: Jean-Baptiste Lallement <jean-baptiste.lallement@canonical.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# TODO:
#    - How to identify device specific issues
#    - How to identify release specific issues

import argparse
import configparser
import os
import sys
import logging
from launchpadlib.launchpad import Launchpad

import practitest
from pprint import pprint

FILTER_OPENED = 51493
FILTER_REPORTED = 52065
FILTER_NOT_REPORTED = 52066

FIELD_LPID = '___f_9763'   # PT internal id for LP id

LP = None
PT = None

# Map LP -> PT statuses
LP2PT_STATUS = {
    "New": "new",
    "Opinion": "closed",
    "Invalid": "rejected",
    "Won't Fix": "closed",
    "Expired": "closed",
    "Confirmed": "opened",
    "Triaged": "opened",
    "In Progress": "assigned",
    "Fix Committed": "fixed",
    "Fix Released": "closed",
    "Incomplete": "opened",
}

# When mapping several task status from LP to a single task status in PT, the
# lowest index in this list takes precedence other the rest.
PT_STATUS_ORDER = ['new', 'opened', 'assigned', 'fixed', 'closed', 'rejected']
LP_STATUS_OPEN = ['New', 'Confirmed', 'Triaged', 'In Progress', 'Fix Committed', 'Incomplete']

IMPORTANCE = [
    "Critical",
    "High",
    "Medium",
    "Low",
    "Wishlist",
    "Undecided",
    "Unknown",
]


def set_logging(debugmode=False):
    """Initialize logging"""
    logging.basicConfig(
        level=logging.DEBUG if debugmode else logging.INFO,
        format="%(asctime)s %(levelname)s %(message)s")
    logging.debug('Debug mode enabled')


def lpinit():
    global LP
    LP = Launchpad.login_with('lp2pt', 'production')


def _parse_arguments():
    """Parse command-line args, returning an argparse dict."""
    parser = argparse.ArgumentParser(
        'Sync issues between Launchpad and Practitest.')
    parser.add_argument('-d', '--debug', action='store_true',
                        default=False, help='enable debug mode')
    return parser.parse_args()


def _read_config_for_var(var_name, config_file):
    """Retrieve the config var, defaulting to an env var when set."""
    try:
        return os.environ[var_name]
    except KeyError:
        try:
            return config_file['DEFAULT'][var_name]
        except KeyError:
            raise KeyError(
                ('{} not specified in config or '
                 'by environment variable.').format(var_name))


def _read_config(config_filepath):
    """Parse the config at the given filepath, returning a config dict."""
    config_file = configparser.ConfigParser()
    config_file.read(config_filepath)
    config = {}
    for var_name in ('PRACTITEST_PROJECT_ID',
                     'PRACTITEST_API_KEY',
                     'PRACTITEST_API_SECRET_KEY',
                     'PRACTITEST_USER_EMAIL',
                     'LP_SUBSCRIBE'):
        new_var_name = var_name.lower().replace('practitest_', '')
        config[new_var_name] = _read_config_for_var(var_name,
                                                    config_file)
    return config


def map_importance(tasks):
    """ Map several tasks importance in LP to 1 importance in PT """

    pt_lowest_idx_opened = 99
    pt_lowest_idx_closed = 99
    for task in tasks:
        if task.status in LP_STATUS_OPEN:
            pt_lowest_idx_opened = min(
                IMPORTANCE.index(task.importance),
                pt_lowest_idx_opened)
        else:
            pt_lowest_idx_closed = min(
                IMPORTANCE.index(task.importance),
                pt_lowest_idx_closed)
    if pt_lowest_idx_opened < 99:
        return IMPORTANCE[pt_lowest_idx_opened]
    else:
        return IMPORTANCE[pt_lowest_idx_closed]


def map_status(tasks):
    """ Map several tasks status in LP to 1 status in PT """
    pt_lowest_idx = 99
    for task in tasks:
        pt_status = LP2PT_STATUS[task.status]
        pt_lowest_idx = min(PT_STATUS_ORDER.index(pt_status), pt_lowest_idx)

    return PT_STATUS_ORDER[pt_lowest_idx]


def is_subscribed(bug, person):
    for s in bug.subscriptions:
        if s.person == person:
            logging.debug("User '{}' is already subscribed".format(
                person.display_name))
            return True
    return False


def update_issue(issue, subscriber=None):
    lpid = issue[FIELD_LPID]['value']
    ptid = issue['id']

    if lpid is None:
        logging.debug('issue {} has no LP Id, skipping'.format(issue['id']))
        return

    logging.debug('Processing issue {}/lp:{}'.format(ptid, lpid))
    bug = LP.bugs[lpid]

    # For duplicate use master bug report
    if bug.duplicate_of is not None:
        bug = bug.duplicate_of
        lpid = bug.id

    status = None
    importance = None
    tasks_desc = "<h3>Bug Tasks</h3>"
    tasks_desc += '<ul>'
    for task in bug.bug_tasks:
        tasks_desc += '<li>{}&nbsp;&nbsp;{}&nbsp;&nbsp;{}&nbsp;&nbsp;</li>'.format(
            task.bug_target_display_name,
            task.status,
            task.importance)
    tasks_desc += '</ul>'
    description = "{}\n\n{}".format(bug.description, tasks_desc)
    status = map_status(bug.bug_tasks)
    importance = map_importance(bug.bug_tasks)

    ptrc = PT.update_issue(ptid, lpid, bug.title, description, status, importance)
    if subscriber is not None:
        if not is_subscribed(bug, subscriber):
            logging.debug('Subscribing {}'.format(subscriber.display_name))
            try:
                bug.subscribe(person=subscriber)
            except:
                logging.warning('Failed to subscribe user or user already subscribed')

    return ptrc


def main():
    """ Main Routine """
    global PT

    args = _parse_arguments()
    set_logging(args.debug)
    lpinit()

    config = _read_config(
        os.path.join(os.path.dirname(__name__),
                     'config.ini'))
    subscriber = LP.people[config['lp_subscribe']]
    PT = practitest.PractitestSession(
        config['project_id'],
        config['api_key'],
        config['api_secret_key'],
        config['user_email'])
    issues = PT.get_issues(FILTER_REPORTED)

    print('Number of issues found: {}'
          .format(issues['additional_data']['pagination']['total_entities']))

    for issue in issues['data']:
        update_issue(issue, subscriber)


if __name__ == '__main__':
    sys.exit(main())
