
# PractiSubunit
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import queue
import tempfile
import threading

from qakit.practitest import subunit_results
from qakit.practitest import practitest
from qakit.practitest.util import logger, queue_to_list


class PractiSubunitSession(practitest.PractitestSession):

    def upload_test_result(self, result):
        logger.info('Reporting result for {}.'.format(result['test_name']))
        exit_code = int(result['subunit_test_result']['status'] != 'success')
        instance_display_id = result['practitest_instance']['id']
        log_files = []
        details = result.get('subunit_test_result').get('details')
        if details:
            tempdir = tempfile.mkdtemp()
            for log_file in details:
                with open(os.path.join(tempdir, os.path.basename(log_file)),
                          'wb') as tmplog:
                    for chunk in details[log_file].iter_bytes():
                        tmplog.write(chunk)
                log_files.append((log_file, tmplog.name))
                tmplog.close()
        super().upload_test_result(exit_code, instance_display_id, log_files)


class PractiSubunitResultSet:

    def __init__(self,
                 practisubunit_session,
                 practitest_testset,
                 subunit_results):
        # yes we need this b/c some digging is required
        self.practisubunit_session = practisubunit_session
        self.results = self._assemble_results(practitest_testset,
                                              subunit_results)

    def _assembly_worker(self,
                         practitest_instances_queue,
                         practitest_tests,
                         subunit_result_list,
                         results_queue):
        """Thread assembling a results dict."""
        result = {}
        try:
            practitest_instance = practitest_instances_queue.get()
            result['practitest_instance'] = practitest_instance
            # NOTE: this is the expensive operation--need the tests
            # list b/c custom fields appear in the *test* only, need
            # to make a relation between instance and test on name
            subunit_test_name = \
                self.practisubunit_session.get_instance_custom_field(
                    practitest_instance,
                    'External ID',
                    practitest_tests)
            if subunit_test_name:
                result['test_name'] = subunit_test_name
                subunit_result = \
                    subunit_results.get_subunit_test_by_name(
                        subunit_test_name,
                        subunit_result_list)
                if subunit_result:
                    result['subunit_test_result'] = subunit_result
                    results_queue.put(result)
        finally:
            practitest_instances_queue.task_done()

    def _assemble_results(self,
                          practitest_testset_number,
                          subunit_result_list):
        """Assemble a simple dict of results.

            {'test_name': _ # the subunit suite (autopilot) test name
             'practitest_instance': _ # practitest dict
             'subunit_test_result': _ # subunit dict
            }

        """
        practitest_instances = self.practisubunit_session.get_instances(
            practitest_testset_number)
        practitest_tests = self.practisubunit_session.get_tests()
        logger.info('Retrieving relevant PractiTest results. . . .')

        practitest_instances_queue = queue.Queue()
        practisubunit_results_queue = queue.Queue()
        for practitest_instance in practitest_instances:
            thread = threading.Thread(
                target=self._assembly_worker,
                args=(practitest_instances_queue,
                      practitest_tests,
                      subunit_result_list,
                      practisubunit_results_queue)
            )
            thread.setDaemon(True)
            thread.start()
            practitest_instances_queue.put(practitest_instance)
        practitest_instances_queue.join()

        return queue_to_list(practisubunit_results_queue)

    def report_to_practitest(self):
        for result in self.results:
            # PractiTest can only handle absolute passes and fails with the
            # automated tests API
            if result['subunit_test_result']['status'] in ('success', 'fail'):
                self.practisubunit_session.upload_test_result(result)
