
# PractiSubunit
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import datetime
import json
import logging
import os

import requests

from pprint import pformat

from qakit.practitest import auth
from qakit.practitest import util


PROD_PRACTITEST = 'https://prod.practitest.com/api'

TEST_LEVEL = '___f_10490'
AUTOMATED = '___f_10534'
EXTERNAL_ID = '___f_10589'


def practitest_time_to_datetime(
        practitest_time, time_format='%Y-%m-%d %H:%M:%S'):
    """Convert from PractiTest time to Python datetime.

    PractiTest times look like '2015-04-13 22:02:03'.

    """
    return datetime.datetime.strptime(
        practitest_time, time_format)


def append_last_run_datetime(practitest_dict):
    """Append a Python datetime for 'last_run' to a given PractiTest dict.

    PractiTest gives us its datetimes as strings; translating to a Python
    datetime makes querying, sorting, etc. much easier.

    :param practitest_dict: a PractiTest JSON dict such as an instance or a
        testset

    """
    try:
        practitest_dict['last_run_datetime'] = practitest_time_to_datetime(
            practitest_dict['last_run']['value'])
    except ValueError:
        # no last_run, no harm done
        pass
    except TypeError:
        # sometimes we get 'last_run' of None, not really a problem
        pass
    return practitest_dict


class PractitestSession:

    def __init__(self, project_id, api_key, api_secret_key, user_email=None):
        self.project_id = project_id
        self.api_key = api_key
        self.api_secret_key = api_secret_key
        self.user_email = user_email

    def pt_field(self, name):
        """Maps PT fields to their IDs"""
        PT_FIELDS = {'device': 10476,
                     'level': 10490,
                    'release': 10433,
                    'build': 10578,
                    'buildinfo': 10872,
                    'external_id': 10589,
                    'tester': 30429
        }
        return '___f_{}'.format(PT_FIELDS[name])

    @util.retry(requests.exceptions.ConnectionError,
                delay=1,
                logger=util.logger)
    def _get(self, url, params={}):
        """Get the given url, retrying on failure."""
        params['project_id'] = self.project_id
        response = requests.get(
            url=url,
            headers=auth.compose_headers(
                self.api_key,
                self.api_secret_key),
            data=json.dumps(params))
        response.raise_for_status()
        return response


    def _get_all(self, url, params={}):
    
        def needs_new_page(pagination):
            total_read = int(pagination['page']) * int(pagination['limit'])
            return int(pagination['total_entities']) - total_read > 0

        json_data = []
        response = None
        params['page'] = 1
        while True:
            response = self._get(url, params)
            if not response.ok:
                break
            json_data.extend(response.json()['data'])
            pagination = response.json().get('additional_data').get('pagination')
            if needs_new_page(pagination):
                params['page'] = str(pagination['page'] + 1)
            else:
                break
        response.raise_for_status()
        return json_data


    @util.retry(requests.exceptions.ConnectionError,
                delay=1,
                logger=util.logger)
    def _post(self, url, data, files=[]):
        """Post the given data to the given url, retrying on failure."""
        data['project_id'] = self.project_id
        logging.debug(
            'Posting to {} with data {} and files {}'.format(
                url, data, files
            )   
        )
        content_type = None
        if not files:
            content_type = 'application/json'
            data = json.dumps(data)
        return requests.post(
            url=url,
            data=data,
            files=files,
            headers=auth.compose_headers(
                self.api_key,
                self.api_secret_key,
                content_type=content_type
            )
        )

    @util.retry(requests.exceptions.ConnectionError,
                delay=1,
                logger=util.logger)
    def _put(self, url, data):
        """Put the given data to the given url, retrying on failure."""
        data['project_id'] = self.project_id
        logging.debug('PUT %s with data %s', url, data)
        req = requests.put(
            url=url,
            data=json.dumps(data),
            headers=auth.compose_headers(
                self.api_key,
                self.api_secret_key
            )
        )
        return req

    def get_tests(self, filter_id=None):
        """Return a list of tests.

        Retrieves all tests associated with account.

        See http://support.practitest.com/customer/portal/articles/1108202-tests-api .  # NOQA

        """
        url = PROD_PRACTITEST + '/tests.json'
        params = {}
        if filter_id:
            params['filter_id'] = filter_id
        return self._get_all(url, params)

    def get_test(self, id):
        """Return a test given its ID.

        See http://support.practitest.com/customer/portal/articles/1108202-tests-api .  # NOQA

        :param id: Practitest test id.

        """
        url = PROD_PRACTITEST + '/tests/{}.json'.format(id)
        return self._get(url).json()

    def get_testset(self, id):
        """Return a testset given its id.

        See http://support.practitest.com/customer/portal/articles/1140223-testsets-api .  # NOQA

        :param id: PractiTest testset id, found in page title.

        """
        url = PROD_PRACTITEST + '/sets/{}.json'.format(id)
        try:
            return self._get(url).json()
        except requests.exceptions.HTTPError as e:
            if "TestSet was not found" in e.response.text:
                raise ValueError("TestSet {} not found!".format(id))
            else:
                raise
            

    def create_testset(self, name, priority, device, 
                       level, release, build, buildinfo):
        """
        Create a test set with the name and priority provided.

        :param name: The name of the test set
        :param priority: The priority of the test set
        """
        url = PROD_PRACTITEST + '/sets.json'
        params = {'name': name, 'priority': {'value': priority},
                  self.pt_field('device'): {'value': device},
                  self.pt_field('level'): {'value': level},
                  self.pt_field('release'): {'value': release},
                  self.pt_field('build'): {'value': build},
                  self.pt_field('buildinfo'): {'value': buildinfo}}
        return self._post(url, data=params).json()

    def get_instances(self, id):
        """Return instances given a testset's id.

        See http://support.practitest.com/customer/portal/articles/1140223-testsets-api .  # NOQA

        :param id: PractiTest testset ID, found in page title.

        """
        url = PROD_PRACTITEST + '/sets/{}/instances.json'.format(id)
        try:
            return self._get_all(url)
        except requests.exceptions.HTTPError as e:
            if "TestSet was not found" in e.response.text:
                raise ValueError("TestSet {} not found!".format(id))
            else:
                raise

    def create_instances(self, set_id, test_ids):
        """
        Create instances of specified tests in the specified test set

        :param set_id: The user visible ID of the test set to add the
                       instances to as shown in the test set URL.
        :param test_ids: A list of the system IDs for the test to add
                         instances of.
        :returns The response
        """
        url = PROD_PRACTITEST + '/sets/{}/instances.json'.format(set_id)
        test_ids_json = [{'test_system_id': id} for id in test_ids]

        params = {'bulk_data': test_ids_json}
        return self._post(url, data=params)

    def upload_test_result(
            self,
            exit_code,
            instance_display_id,
            logs=[]):
        """Report the result of one test instance to PractiTest.

        See http://support.practitest.com/customer/portal/articles/360718-automated-tests-api .  # NOQA

        :param exit_code: "0 for passed, otherwise failed".
        :param instance_display_id: "the display_id of an instance".

        """
        url = PROD_PRACTITEST + '/automated_tests/upload_test_result.json'
        result = {"instance_display_id": instance_display_id,
                  "exit_code": exit_code
        }
        if len(logs) > 0:
            result["result"] = "See attached logs for more information."
        upload_logs = {}
        for i in range(len(logs)):
            upload_logs["result_files[file{}]".format(i)] = (
                logs[i][0], open(logs[i][1], 'rb')
            )
        return self._post(url, data=result, files=upload_logs)

    def get_test_from_tests_by_name(self, test_name, tests):
        """Return a test from a list of tests which matches by name.

        :param test_name: the name of test to retrieve.
        :param tests: the list of tests from which to retrieve.

        """
        try:
            return [test for test in tests
                    if test['name'] == test_name][0]
        except IndexError:
            return None

    def _get_id_of_test_by_name(self, test_name, tests):
        """Return the id of a test, finding it by name in a list of tests.

        :param test_name: name of test the id of which to retrieve.
        :param tests: list of tests from which to retrieve.

        """
        try:
            return self.get_test_from_tests_by_name(
                test_name, tests)['display_id']
        except TypeError:
            return None

    def get_test_custom_field(self, test, custom_field_name):
        """Get a custom field of a given test, e.g. 'External ID'.

        Custom fields are packed at a lower level so it's necessary to go
        hunting.

        :param test: the PractiTest test to search.
        :param custom_field_name: e.g. 'External ID'.

        """
        for key, val in test.items():
            try:
                if val['name'] == custom_field_name:
                    # 'None' for blank entries
                    return val['value']
            except TypeError:
                # for shallower fields this could be int, e.g.
                pass
        return None

    def get_instance_custom_field(self, instance, custom_field_name, tests):
        """Return

        :param instance: PractiTest instance.
        :param custom_field_name: FIXME.
        :param tests: list of tests from which to retrieve.

        """
        test_id = self._get_id_of_test_by_name(instance['name'], tests)
        if test_id is not None:
            test = self.get_test(test_id)
            return self.get_test_custom_field(test, custom_field_name)
        return None

    def get_issues(self, id):
        """Return issues given a filter id

        See http://support.practitest.com/customer/portal/articles/1140223-testsets-api .  # NOQA

        :param id: PractiTest filter ID, found in page title.

        """
        url = PROD_PRACTITEST + '/issues.json?filter_id={}'.format(id)

        return self._get(url).json()

    def update_issue(self, ptid, lpid, title, description, status, importance):
        """Report the result of one test instance to PractiTest.

        See http://support.practitest.com/customer/portal/articles/360718-automated-tests-api .  # NOQA

        :param id: Issue Id in practitest
        :param title: Bug title
        :param description: Bug description
        :param status: Bug status
        :param importance: Bug importance

        """
        desc = description.replace('\n', '<br>')
        url = PROD_PRACTITEST + '/issues/{}.json'.format(ptid)  # NOQA

        result = {'___f_9763': {'value': lpid},  # Custom field LP ID
                  'title': title,
                  'user': self.user_email,
                  'description': desc,
                  'issue_status': {'value': status},
                  'priority': {'value': importance},
                  }
        logging.debug(pformat(result))
        logging.info('Updating issue pt:{} / lp:{}'.format(ptid, lpid))
        return self._put(url, data=result)

    @staticmethod
    def get_filters_for_src_pkgs(src_pkgs):
        """
        This method return the list of practitest filter ids associated
        to the src pacckages in the params
        :param src_pkgs: list of source packages
        :return: list of filters
        """
        practitest_dir = os.path.dirname(__file__)
        domains_file = os.path.join(practitest_dir, 'data', 'domains.json')
        with open(domains_file) as domains:
            domains_json = json.loads(domains.read())
        filters = [domains_json[package] for package in src_pkgs]

        # Flat the lists and remove duplicates
        return list(set([item for sublist in filters for item in sublist]))
