#!/usr/bin/python3
# UESQA Metrics
# Copyright (C) 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from datetime import datetime
import logging
import os
import socket
import urllib.request
import time

logger = logging.getLogger(__name__)


class CarbonConnector:
    """ Class used to connect to carbon """

    CARBON_SERVER = os.getenv('CARBON_SERVER', '127.0.0.1')
    CARBON_PORT = int(os.getenv('CARBON_PORT', '2003'))
    MSG_CHUNK_SIZE = int(os.getenv('MSG_CHUNK_SIZE', '500'))
    MSG_WAIT = int(os.getenv('MSG_WAIT', '30'))

    def __init__(self):
        self.urls = {}

    def is_defined(self):
        """ Indicate if variables are ready to connect to carbon """
        server = os.getenv('CARBON_SERVER')
        port = os.getenv('CARBON_PORT')
        return server and port

    def send_msg(self, message):
        """ Send a message to the carbon server """
        logger.info('Sending message to carbon in {}:{}'.format(self.CARBON_SERVER,
                                                                self.CARBON_PORT))
        logger.info(message)
        sock = socket.socket()
        sock.connect((self.CARBON_SERVER, self.CARBON_PORT))
        sock.sendall(message.encode())
        sock.close()

    def _get_chunks(self, metrics, chunck):
        for i in range(0, len(metrics), chunck):
            yield metrics[i:i+chunck]

    def send_metrics(self, metrics):
        """ Send a list of metrics to the carbon server by chuncks.
        Are used the environment variables to send the message to carbon """
        for chunk in self._get_chunks(metrics, self.MSG_CHUNK_SIZE):
            number = len(chunk)
            message = '\n'.join(chunk) + '\n'
            logger.info('Sending chunk of {} metrics'.format(number))
            self.send_msg(message)
            logger.info('Waiting: {}s'.format(self.MSG_WAIT))
            time.sleep(self.MSG_WAIT)

    def _get_build_url(self, channel, device, build_number, opt=1):
        if opt == 1:
            base_url = 'http://people.canonical.com/~lzemczak/landing-team'
            return '{}/{}/{}/{}.commitlog'.format(base_url, channel, device,
                                                  build_number)
        else:
            base_url = 'https://system-image.ubuntu.com'
            return '{}/{}/{}/version-{}.json.asc'.format(base_url, channel,
                                                         device, build_number)

    def get_build_timestamp(self, channel, device, build_number):
        """ Get a timestamp for a build """
        full_url_1 = self._get_build_url(channel, device, build_number)
        if full_url_1 in self.urls.keys():
            return self.urls[full_url_1]

        full_url_2 = self._get_build_url(channel, device, build_number, opt=2)
        if full_url_2 in self.urls.keys():
            return self.urls[full_url_2]

        # Date with format 'Sat, 02 Jul 2016 03:46:00 GMT'
        try:
            logger.info('trying url {}'.format(full_url_1))
            str_date = urllib.request.urlretrieve(full_url_1)[1]['Last-Modified']
            full_url = full_url_1
        except Exception as e:
            logger.info('trying url {}'.format(full_url_2))
            str_date = urllib.request.urlretrieve(full_url_2)[1]['Last-Modified']
            full_url = full_url_2
        timestamp = int(time.mktime(datetime.strptime(
            str_date, "%a, %d %b %Y %H:%M:%S GMT").timetuple()))
        self.urls[full_url] = timestamp
        return timestamp
