#!/usr/bin/python3
# UESQA Metrics
# Copyright (C) 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
import os
from pymongo import MongoClient

logger = logging.getLogger(__name__)


class DAO:
    """ This DAO class is used to interact with the perf test db """

    MONGO_HOST = os.getenv('MONGO_HOST', '127.0.0.1')
    MONGO_PORT = int(os.getenv('MONGO_PORT', '27017'))

    def __init__(self):
        logger.info('Connecting to mongo db in {}:{}'.format(self.MONGO_HOST,
                                                             self.MONGO_PORT))
        self.client = MongoClient(host=self.MONGO_HOST, port=self.MONGO_PORT)
        self.db = self.client.test
        logger.info('Connection to mongo db stablished')

    def __del__(self):
        self.client.close()

    def _get_col(self, col):
        try:
            return self.db.get_collection(col)
        except TypeError:
            return self.db.create_collection(col)

    def is_match(self, elem, value, col='perf'):
        """ Detect if there is a match of the args
        :param elem: elem used to match
        :param value: the value used to compare
        :param col: the collection to be used
        """
        return self._get_col(col).find({elem: value}).count() > 0

    def find(self, args, distinct=None, col='perf'):
        """ Find elements in the db
        :param args: dict with the parameters used to search
        :param distinct: the element used to get distincts
        :param col: the collection to be used
        """
        if distinct:
            return self._get_col(col).find(args).distinct(distinct)
        else:
            return [elem for elem in self._get_col(col).find(args)]

    def find_one(self, args, col='perf'):
        """ Find one element in the db
        :param args: dict with the parameters used to search
        :param col: the collection to be used
        """
        return self._get_col(col).find_one(args)

    def aggregate(self, match, group, col='perf'):
        """ Find elements in the db using aggregation
        :param match: dict with the match used to search
        :param group: dict with the args to group data used to search
        :param col: the collection to be used
        """
        return self._get_col(col).aggregate([match, group])

    def insert(self, elems, col='perf'):
        """ Insert elements to the db
        :param elems: either an element or list of elements to insert in the db
        :param col: the collection to be used
        """
        if isinstance(elems, list):
            self._get_col(col).insert_many(elems)
        else:
            self._get_col(col).insert_one(elems)

    def delete(self, args, col='perf'):
        """ Insert euther an element or a list
        :param args: dict with the parameters used to search and delete
        :param col: the collection to be used
        """
        ids = self.find(args, distinct='_id', col=col)
        logger.info('Deleting {} elems'.format(len(ids)))
        self._get_col(col).delete_many({'_id': {'$in': ids}})
