#!/usr/bin/python3
# UESQA Metrics
# Copyright (C) 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import ast
import logging
import os

logger = logging.getLogger(__name__)


class PerfLogsParser:
    """ Parser used to read logs with time results for performance """

    GLOBAL_DELIMITER = '<='
    TIME_DELIMITER = '=>'

    def parse_log_times(self, log_lines, times_class):
        """ Retrieve the global run information and a list with all the history
        instances parsed from the results_list in the subunit test log
        :param log_lines: the lines in the log
        :param times_class: the class used to filter the times to consider
        :return: the global information of the run and the times for this run
        """
        tests_times = []
        global_info = self._get_global_information(log_lines)

        times = self._get_times(log_lines, times_class)
        if global_info and times:
            tests_times.extend(times)
        return global_info, tests_times

    def _read_log_lines(self, log_file, encoding=None):
        with open(log_file, 'r', encoding=encoding) as file:
            return file.read().splitlines()

    def parse_log_file(self, log_file):
        """ Return a list of lines for the log """
        if not os.path.exists(log_file):
            raise RuntimeError('Log file does not exist {}'.format(log_file))
        try:
            return self._read_log_lines(log_file)
        except UnicodeDecodeError:
            return self._read_log_lines(log_file, encoding='utf-8')

    def _get_global_information(self, log_lines):
        markers = [line for line in log_lines if self.GLOBAL_DELIMITER in line]
        for marker in markers:
            values = marker.split(self.GLOBAL_DELIMITER)[1].strip()
            try:
                return ast.literal_eval(values)
            except (ValueError, TypeError):
                continue
        raise RuntimeError('Global info not provided for tests')

    def _get_times(self, test_log, times_class):
        markers = [line for line in test_log if self.TIME_DELIMITER in line]
        times = []

        for mark in markers:
            values = mark.split(self.TIME_DELIMITER)[1].strip()
            try:
                values_dict = ast.literal_eval(values)
                if 'class' in values_dict and values_dict['class'] == times_class:
                    times.append(values_dict)
            except (ValueError, TypeError):
                logger.error('Error parsing values in line: {}'.format(mark))
                continue

        return times


class PyUnitParser:
    """ Class used to retrieve results from pyunit file """

    DELIMITER = '...'
    PASS = 'ok'
    FAIL = 'FAIL'

    def get_numeric_result(self, result):
        if result == self.PASS:
            return 1
        elif result == self.FAIL:
            return 0
        else:
            return -1

    def get_pyunit_results(self, pyunit):
        with open(pyunit, 'r') as f:
            lines = f.readlines()

        results = {}
        for line in lines:
            if self.DELIMITER in line:
                parts = line.split(self.DELIMITER)
                if len(parts) == 2 and self.get_numeric_result(parts[1].strip()) >= 0:
                    results[parts[0].strip()] = self.get_numeric_result(parts[1].strip())

        return results
