#!/usr/bin/python3
# UESQA Metrics
# Copyright (C) 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from argparse import ArgumentParser
from datetime import datetime
import json
import logging
import os
import time

from qakit.prodenv.engine.carbon import CarbonConnector
from qakit.prodenv.engine.dao import DAO

logger = logging.getLogger(__name__)


class ISOSync:
    """ Class used to sync iso data with mongo and perf """

    BASE_METRIC = 'tests.iso'
    JSON_SUMMARY = 'summary.json'
    COLLECTION = 'iso'

    def __init__(self):
        self.dao = DAO()
        self.connector = CarbonConnector()

    def identify_iso_name(self, filename):
        """ Recognized files with the following format yakkety-desktop-amd64.iso """
        logger.info('Processing iso file: {}'.format(filename))
        return filename.split('.')[0].split('-')

    def get_grafana_metric(self, build_id, version, device, arch, key, value):
        timestamp = int(time.mktime(datetime.strptime(build_id, "%Y%m%d").timetuple()))
        metric = '{}.{}.{}.{}.{}.{} {} {}'.format(self.BASE_METRIC, version, device, arch,
                                                  key, build_id, value, timestamp)
        return metric

    def get_mongo_metric(self, type, build_id, version, device, arch, key, value):
        return {'class': 'iso', 'type': type, 'version': version, 'device': device,
                'build_id': build_id, 'arch': arch, 'key': key, 'value': value}

    def send_to_grafana(self, summary, build_id, version, device, arch):
        if not self.connector.is_defined():
            logger.error('Server data not provided, sending data to detault')

        metrics = []
        for key, value in summary.items():
            metrics.append(self.get_grafana_metric(build_id, version, device, arch, key, value))

        message = '\n'.join(metrics) + '\n'
        self.connector.send_msg(message)

    def send_to_mongo(self, summary, build_id, version, device, arch):

        metrics = []
        for key, value in summary.items():
            metrics.append(self.get_mongo_metric('disk_footprint', build_id, version, device, arch, key, value))

        self.dao.insert(metrics, col=self.COLLECTION)

    def sync_summary(self, summarydir, iso_file):
        """ Sync with perf. It is done just when env variables are set """
        json_file = os.path.join(summarydir, self.JSON_SUMMARY)
        if not os.path.exists(json_file):
            logger.error('Summary data not provided on: {}'.format(os.path.join(summarydir, self.JSON_SUMMARY)))
            return

        with open(json_file) as file:
            summary = json.load(file)

        (version, device, arch) = self.identify_iso_name(os.path.basename(iso_file))
        build_id = summary.pop('build_id')

        self.send_to_grafana(summary, build_id, version, device, arch)
        self.send_to_mongo(summary, build_id, version, device, arch)


if __name__ == "__main__":
    """Parse command-line args, returning an argparse dict."""
    parser = ArgumentParser('Parser for footprint sync with perf and mongodb')
    parser.add_argument('isofile', help='Path to a valid desktop ISO')
    parser.add_argument('summarydir', help='Dir where the summary is stored')
    args = parser.parse_args()

    logger.info('Starting iso footprint sync')
    sync = ISOSync()
    sync.sync_summary(args.summarydir, args.isofile)
