#!/usr/bin/python3
# UESQA Metrics
# Copyright (C) 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from argparse import ArgumentParser
import csv
import logging
import os

from qakit.prodenv.engine.dao import DAO

logger = logging.getLogger(__name__)


class PerfReporter:
    """ Class used to create reports using information retrieved
    from the performance collection
    """

    FIELDNAMES = ['class', 'type', 'name', 'time', 'test', 'load',
                  'device', 'channel', 'build_number', 'run_id']
    NOTINCLUDE = ['_id', '_updated', '_created', '_etag']

    def __init__(self):
        self.dao = DAO()

    def _write_csv_report(self, values, output):
        with open(output, 'w') as csvfile:
            writer = csv.DictWriter(csvfile, fieldnames=self.FIELDNAMES)

            writer.writeheader()
            for x in values:
                for y in self.NOTINCLUDE:
                    x.pop(y)
                writer.writerow(x)

    def _create_filter(self, args):
        filter = {}
        if args.rclass:
            filter['class'] = args.rclass
        if args.type:
            filter['type'] = args.type
        if args.name:
            filter['name'] = args.name
        if args.test:
            filter['test'] = args.test
        if args.load:
            filter['load'] = args.load
        if args.device:
            filter['device'] = args.device
        if args.channel:
            filter['channel'] = args.channel
        if args.build_from:
            filter['build_from'] = {"$gt": args.build_from}
        if args.build_to:
            filter['build_to'] = {"$lt": args.build_to}
        return filter

    def create_csv_report(self, args):
        filter = self._create_filter(args)
        values = self.dao.find(filter)
        self._write_csv_report(values, args.output)


def parse_input():
    args_parser = ArgumentParser("Parse filter parameters.")
    args_parser.add_argument('-c', '--rclass', type=str,
                             help='The class to filter, i.e. perf, '
                                  'app_startup, etc')
    args_parser.add_argument('-t', '--type', type=str,
                             help='The type to filter, i.e. hot, cold, etc')
    args_parser.add_argument('-n', '--name', type=str,
                             help='The name to filter, i.e. reboot, dialer, '
                                  'javascript, etc')
    args_parser.add_argument('-st', '--test', type=str,
                             help='The test to filter, i.e. '
                                  'vertical_scrolling, horizontal_scrolling, '
                                  'etc')
    args_parser.add_argument('-l', '--load', type=int,
                             help='The load to filter')
    args_parser.add_argument('-d', '--device', type=str,
                             help='The device to filter, i.e. krillin, arale, '
                                  'etc')
    args_parser.add_argument('-ch', '--channel', type=str,
                             help='The channel to filter, i.e. '
                                  'ubuntu-touch/rc-proposed/bq-aquaris.en, '
                                  'etc')
    args_parser.add_argument('-bf', '--build_from', type=int,
                             help='From which build number to filter')
    args_parser.add_argument('-bt', '--build_to', type=int,
                             help='To which build number to filter')
    args_parser.add_argument('-o', '--output', type=str, default='out.csv',
                             help='To which build number to filter')
    return args_parser.parse_args()


if __name__ == "__main__":
    args = parse_input()
    if os.path.exists(args.output):
        logger.info('Output file {} already exists'.format(args.output))
        exit(1)

    reporter = PerfReporter()
    reporter.create_csv_report(args)
