#!/usr/bin/python3
# UESQA Metrics
# Copyright (C) 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from argparse import ArgumentParser
import logging
from urllib.error import HTTPError

from qakit.prodenv.engine.carbon import CarbonConnector
from qakit.prodenv.engine.dao import DAO
from qakit.prodenv.engine.parser import PerfLogsParser
from qakit.prodenv.perf.processor import DataProcessor

logger = logging.getLogger(__name__)


class PerfSync:
    """ Class used to integrate perf metrics within perf """

    TIME_DELTA = 300
    CLASS = 'perf'

    def __init__(self):
        self.timestamps = {}
        self.dao = DAO()
        self.connector = CarbonConnector()
        self.processor = DataProcessor()

    def _get_aggregation_match(self, device=None, channel=None, build_number=None):
        match = {'class': self.CLASS}
        if device:
            match['device'] = device
        if channel:
            match['channel'] = channel
        if build_number:
            match['build_number'] = build_number
        return {'$match': match}

    def _get_aggregation_group(self):
        return {'$group': {'_id': {'class': '$class',
                                   'type': '$type',
                                   'name': '$name',
                                   'build_number': '$build_number',
                                   'test': '$test',
                                   'device': '$device',
                                   'channel': '$channel'},
                           'avgtime': {'$avg': '$time'}}}

    def _get_metric(self, elem):
        separator = '.'
        metric = ''
        metric += elem.get('_id').get('class') + separator
        metric += elem.get('_id').get('device') + separator
        metric += elem.get('_id').get('channel').split('/')[1] + separator
        metric += elem.get('_id').get('name') + separator
        metric += elem.get('_id').get('type') + separator
        metric += (elem.get('_id').get('test') or 'default') + separator
        metric += str(elem.get('_id').get('build_number'))
        return metric

    def _gen_metrics(self, elems):
        metrics = []
        for elem in elems:
            metric = self._get_metric(elem)
            timestamp = self.connector.get_build_timestamp(elem.get('_id').get('channel'),
                                                           elem.get('_id').get('device'),
                                                           elem.get('_id').get('build_number'))

            if metric in self.timestamps.keys():
                timestamp = self.timestamps[metric] + self.TIME_DELTA
            self.timestamps[metric] = timestamp

            value = round(elem.get('avgtime'), 4)
            metrics.append('{} {} {}'.format(metric, value, timestamp))
        return metrics

    def _save_to_mongo_db_if_needed(self, global_info, times):
        if times:
            run_id = global_info['run_id']
            if not self.dao.is_match('run_id', run_id):
                self._save_record_to_mongo_db(global_info, times)
            else:
                logger.warning(
                    'Test results already in the db for the run_id {}'.format(
                        run_id))

    def _save_record_to_mongo_db(self, global_info, records):
        logger.info('Saving records to db')
        updated_records = []
        for record in records:
            record.update(global_info)
            updated_records.append(record)
        self.dao.insert(updated_records)

    def migrate_full_mongo_db(self):
        """ Migrate the full mongo database performance collection to grafana """
        builds = sorted(self.dao.find({'class': self.CLASS},
                                 distinct='build_number'))
        logger.info('Builds to be migrated: {}'.format(builds))
        for build in builds:
            results = self.dao.aggregate(self._get_aggregation_match(build_number=build),
                                         self._get_aggregation_group())
            try:
                metrics = self._gen_metrics(results)
                metrics = self.processor.process(metrics)

                if not self.connector.is_defined():
                    logger.error('Server data not provided, sending data to detault')
                logger.info('Sending total of {} metrics'.format(len(metrics)))
                self.connector.send_metrics(metrics)
            except HTTPError:
                logger.info('Skiping build: {}'.format(build))

            self.timestamps = {}

    def migrate_from_mongo_db(self, build, device, channel):
        """ Migrate performance results from mongo db to grafana for an specific execution """
        results = self.dao.aggregate(self._get_aggregation_match(device=device,
                                                                 channel=channel,
                                                                 build_number=build),
                                     self._get_aggregation_group())
        metrics = self._gen_metrics(results)
        metrics = self.processor.process(metrics)

        if not self.connector.is_defined():
            logger.error('Server data not provided, sending data to detault')
        logger.info('Sending total of {} metrics'.format(len(metrics)))
        self.connector.send_metrics(metrics)

    def save_to_mongo_db(self, log_file):
        """ Save performance results from test log to mongo db """
        parser = PerfLogsParser()
        log_lines = parser.parse_log_file(log_file)
        (global_info, startup_times) = parser.parse_log_times(log_lines, self.CLASS)
        self._save_to_mongo_db_if_needed(global_info, startup_times)


def check_update_args(remaining_argv):
    args_parser = ArgumentParser("Parse input parameters")
    args_parser.add_argument('-b', '--build', type=int, required=True,
                             help='The build number to migrate')
    args_parser.add_argument('-c', '--channel', type=str, required=True,
                             help='The channel to migrate')
    args_parser.add_argument('-d', '--device', type=str, required=True,
                             help='The device to migrate')
    return args_parser.parse_args(args=remaining_argv)


def check_save_args(remaining_argv):
    args_parser = ArgumentParser("Parse input parameters")
    args_parser.add_argument("log")
    return args_parser.parse_args(args=remaining_argv)


if __name__ == '__main__':
    args_parser = ArgumentParser("Parse input parameters")
    args_parser.add_argument('action', choices=['migrate', 'update', 'save'],
                             help='Action to do')
    args, remaining_argv = args_parser.parse_known_args()
    sync = PerfSync()

    if args.action == 'migrate':
        sync.migrate_full_mongo_db()
    elif args.action == 'update':
        new_args = check_update_args(remaining_argv)
        sync.migrate_from_mongo_db(new_args.build, new_args.device, new_args.channel)
    elif args.action == 'save':
        new_args = check_save_args(remaining_argv)
        sync.save_to_mongo_db(new_args.log)
