#!/usr/bin/python3
# UESQA Metrics
# Copyright (C) 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from argparse import ArgumentParser
import logging
import os

from qakit.prodenv.engine.carbon import CarbonConnector
from qakit.prodenv.engine.dao import DAO
from qakit.prodenv.engine.parser import PyUnitParser

logger = logging.getLogger(__name__)


class TestsSync:
    """ Class used to sync tests results with mongo and grafana """

    COLLECTION = 'tests'
    TEST_PARTS = 3

    def __init__(self):
        self.dao = DAO()
        self.connector = CarbonConnector()

    def send_to_mongo(self, results, channel, build, device, jobid=''):
        metrics = []
        base_matric = {'class': 'test_result', 'channel': channel, 'build_number': int(build),
                       'device': device, 'job_id': jobid}
        for test, result in results.items():
            result = {'test': test, 'result': result}
            result.update(base_matric)
            metrics.append(result)

        self.dao.insert(metrics, col=self.COLLECTION)

    def get_grafana_metric(self, channel, device, build_number, test, result):
        base_metric = 'tests.test_result'
        timestamp = self.connector.get_build_timestamp(channel, device, build_number)
        short_channel = channel.split('/')[1]
        short_test = '.'.join(test.split('.')[-self.TEST_PARTS:])
        metric = '{}.{}.{}.{}.{} {} {}'.format(base_metric, device, short_channel,
                                               short_test, build_number, result, timestamp)
        return metric

    def send_to_grafana(self, results, channel, build_number, device):
        if not self.connector.is_defined():
            logger.error('Server data not provided, sending data to detault')

        logger.info('Configuration for perf detected')
        metrics = []
        for test, result in results.items():
            metrics.append(self.get_grafana_metric(channel, device, build_number, test, result))

        message = '\n'.join(metrics) + '\n'
        self.connector.send_msg(message)


def check_args(args):
    if not os.path.isfile(args.pyunit):
        logger.info('Pyunit file not found')
        exit(1)

    try:
        int(args.build)
    except:
        logger.info('Build has to be numeric')
        exit(1)


def parse_args():
    parser = ArgumentParser('Parser for tests sync with mongo db and perf')
    parser.add_argument('-p', '--pyunit', help='Path to a subunit file', required=True)
    parser.add_argument('-c', '--channel', help='Channel used for the tests', required=True)
    parser.add_argument('-b', '--build', help='Build number used for the tests', required=True)
    parser.add_argument('-d', '--device', help='Device used for the device', required=True)
    parser.add_argument('-j', '--jobid', help='Job identifier where the tests have been executed')
    args = parser.parse_args()
    check_args(args)
    return args


if __name__ == "__main__":
    """Parse command-line args, returning an argparse dict."""
    args = parse_args()

    logger.info('Starting test results sync')
    parser = PyUnitParser()
    sync = TestsSync()

    results = parser.get_pyunit_results(args.pyunit)
    sync.send_to_grafana(results, args.channel, args.build, args.device)
    sync.send_to_mongo(results, args.channel, args.build, args.device, args.jobid)
