# Copyright (C) 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import json
import logging

from jenkins import Jenkins
from pika import BlockingConnection
from pika import ConnectionParameters

from qakit.config import Config
from qakit.launchpad.ppa import Ppa
from qakit.practitest.practitest import PractitestSession


class Worker:

    bileto_results = {"PASSED": "Approved", "FAILURE": "Failed", "QUEUED": "Queued"}

    def __init__(self):
        self.conf = Config()
        self.jenkins = Jenkins(self.conf.parser.get('jenkins', 'server'),
                               self.conf.parser.get('jenkins', 'username'),
                               self.conf.parser.get('jenkins', 'password'))

    def run_ust(self, ppa):
        """"
        This method will execute the following steps:
        1. Get the source packages from launchpad that are
        included in the ppa under test
        2. Use the packages versions to create a request id that will be use to track
        the test results for the source packages. The idea is to run the tests for every
        version of the source packages in the ppa
        3. Get the practitest filter id associated to the source packages in the ppa
        4. Get the list of test cases to be run from practitest
        5. Trigger a jenkins build that will:
            a. Flash the device
            b. Update the device with the ppa under test
            c. Setup the device for testing
            d. Run the tests

        :param ppa is the ppa associated to the request in bileto and it is
        expected to be in the format team/ppa_name.
        For example: ci-train-ppa-service/landing-059

        This method has two entry points, it can be called by importing the
        qakit module and creating a Worker instance like this
            Worker().run_ust("ci-train-ppa-service/landing-059")
        And there's also an amqp entry point. The start method will start an amqp
        client which will be subscribed to /ubuntu-system-test, whenever a message
        is published in such a topic, it will be process by the
        process_ust_request method which will call this one to run the tests
        """
        team, ppa_name = ppa.split('/')
        launchpad_ppa = Ppa(team=team, name=ppa_name)
        src_pkgs = launchpad_ppa.get_source_packages()
        packages = list(
            set([package.source_package_name for package in src_pkgs]))
        request_id = launchpad_ppa.calculate_hash()

        pt = PractitestSession(self.conf.parser.get('practitest', 'project_id'),
                               self.conf.parser.get('practitest', 'api_key'),
                               self.conf.parser.get('practitest', 'api_secret_key'))
        filters = pt.get_filters_for_src_pkgs(packages)
        tests = pt.get_tests(filters)

        try:
            tests_ids = ' '.join([test['___f_10589']['value'] for test in tests])
        except KeyError:
            tests_ids = ' '

        self.jenkins.build_job(self.conf.parser.get('jenkins', 'job_name'),
                               {"PPA": ppa, "TESTS_TO_RUN": tests_ids, "REQUEST_ID": request_id},
                               {'token': self.conf.parser.get('jenkins', 'token')})

    def get_result(self, ppa):
        """
        The tests results are retrieved from jenkins and the ppa is used to get the
        appropriate tests results. As when the tests were triggered, the list of
        source packages belonging to the ppa are retrieved from launchpad and their
        versions is used to get the request id associated to those group of packages

        We'll first iterate on the builds returned by the get_job_info and we'll return
        the results associated to the build which request id matches the request id calculated
        by this method.

        If the correct build is not found in the get_job_info results, then we'll check the
        get_queue_info and if the build is not found there, then an error is returned
        """
        team, ppa_name = ppa.split('/')
        launchpad_ppa = Ppa(team=team, name=ppa_name)
        request_id = launchpad_ppa.calculate_hash()

        builds = self.jenkins.get_job_info(
            self.conf.parser.get('jenkins', 'job_name'), depth=2)['builds']
        build = self._get_build_by_request_id(builds, request_id)

        if build:
            build_info = self.jenkins.get_build_info(
                self.conf.parser.get('jenkins', 'job_name'), build['number'])
            return UstResult(self.bileto_results[build_info['result']], build_info['url'])
        else:
            builds = self.jenkins.get_queue_info()
            build = self._get_build_by_request_id(builds, request_id)
            if build:
                return UstResult("QUEUED", "")
            else:
                return None

    @staticmethod
    def _get_build_by_request_id(builds, request_id):
        """
        This method iterates over a list of buils and return the one which
        request id in the parameters matches the request_id
        :param builds: List of builds
        :param request_id: request id to filter the builds
        :return: build
        """
        for build in builds:
            parameters = list(filter(
                lambda action: action.get('parameters', None),
                build['actions']))[0]['parameters']
            try:
                req_id = list(filter(
                    lambda kv: kv['name'] == 'REQUEST_ID', parameters))[0]['value']
                if request_id == req_id:
                    return build
            except IndexError:
                logging.info("Build does not have a request id set")
        return None

    def start(self):
        """
        This method would start an amqp client and subscribe to the queue
        set in the configuration file
        """
        try:
            logging.info('Connecting to AMQP server: {}'.format(
                self.conf.parser.get('amqp', 'host')))
            connection = BlockingConnection(ConnectionParameters(
                host=self.conf.parser.get('amqp', 'host')))
            channel = connection.channel()
            queue = self.conf.parser.get('amqp', 'queue')

            logging.info('Subscribing to queue: {}'.format(
                self.conf.parser.get('amqp', 'queue')))
            channel.queue_declare(queue, durable=True, auto_delete=False)
            channel.basic_consume(
                self.process_ust_request, queue=queue, no_ack=True)
            channel.start_consuming()
        except KeyboardInterrupt:
            channel.stop_consuming()
            connection.close()

    def process_ust_request(self, ch, method, properties, body):
        """Callback for ubuntu system test request"""
        logging.info('Message received {}'.format(body))

        ppa = json.loads(body.decode('utf-8'))['ppa']
        self.run_ust(ppa)


class UstResult:
    def __init__(self, status, build_url):
        self.status = status
        self.build_url = build_url

if __name__ == '__main__':
    Worker().start()
