#!/usr/bin/python3

# QA KPI Utilities
# Copyright (C) 2015-2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import argparse
import logging
import sys

import bson
import jenkinsapi
import pymongo

logger = logging.getLogger(__name__)
handler = logging.StreamHandler()
handler.setLevel(logging.DEBUG)
logger.addHandler(handler)
logger.setLevel(logging.DEBUG)


def filter_job_names(job_names, pattern):
    return list(filter(lambda x: pattern in x, job_names))


class JenkinsMongoSession():

    def __init__(self, db, baseurl, username=None, password=None):
        super().__init__()
        self.db = db
        self.jenkins = jenkinsapi.jenkins.Jenkins(
            baseurl, username, password)

    def get_job(self, job_name):
        return self.jenkins[job_name]

    def retrieve_build_resultset(self, db_build_id):
        """Retrieve the resultset of a known build in our db."""
        db_build = self.db.builds.find_one(
            {'_id': bson.ObjectId(db_build_id)})
        job_name = db_build['job_name']
        jenkins_job = self.get_job(job_name)
        build_number = db_build['number']
        jenkins_build = jenkins_job.get_build(build_number)
        try:
            resultset = jenkins_build.get_resultset()
        except jenkinsapi.custom_exceptions.NoResults:
            return {}
        test_results = []
        for test_name in resultset.keys():
            result = resultset[test_name]
            test_result = {
                'test_name': test_name,
                'status': result.status,
                'build_number': build_number,
                'build_timestamp': db_build['timestamp'],
                'build_name': db_build['description'],
                'job_name': job_name,
            }
            test_results.append(test_result)
        return test_results

    def retrieve_and_insert_build_resultset(self, db_build_id):
        """Retrieve test results from Jenkins, inserting into our db."""
        build_resultset = self.retrieve_build_resultset(db_build_id)
        return self.db.tests.insert(build_resultset)

    def retrieve_build(self, job_name, build_number):
        """Retrieve build from Jenkins."""
        logger.info(
            'Retrieving {} build #{}.'.format(job_name, build_number)
        )
        job = self.get_job(job_name)
        build = job.get_build(build_number)
        data = build.get_data(build.python_api_url(build.baseurl))
        actions = build.get_actions()
        total = failed = skipped = 0
        try:
            total = actions['totalCount']
            failed = actions['failCount']
            skipped = actions['skipCount']
        except KeyError:
            # we don't have a real result here but we'll still save
            pass
        return dict({
            'job_name': job_name,
            'total': total,
            'failed': failed,
            'skipped': skipped},
            **data)

    def retrieve_and_insert_build(self, job_name, build_number):
        """Retrieve build from Jenkins and deposit in db."""
        build_results = self.retrieve_build(job_name, build_number)
        return self.db.builds.insert(build_results)

    def get_db_build_ids_for_job(self, job_name):
        """Get a list of known build ids for the given job for comparison."""
        return [
            build['number'] for build in self.db.builds.find(
                {'job_name': job_name},
                {'number': 1, '_id': 0})
        ]

    def get_missing_builds(self, job_name):
        """Return a list of build ids in Jenkins we're missing in db."""
        job = self.get_job(job_name)
        jenkins_build_ids = set(job.get_build_ids())
        db_build_ids = set(self.get_db_build_ids_for_job(job_name))
        missing_builds = [
            build_id for build_id in
            # take the set-difference
            list(jenkins_build_ids.difference(db_build_ids))
            # filter out running builds
            if not job.get_build(build_id).is_running()]
        logger.info(
            'For job {}, missing builds {}.'.format(job_name, missing_builds))
        return missing_builds

    def retrieve_data_for_job(self, job_name):
        """Retrieve builds and results from Jenkins, inserting into our db."""
        logger.debug('Retrieving data for job {}.'.format(job_name))
        build_numbers = self.get_missing_builds(job_name)
        for build_number in reversed(build_numbers):
            db_build_id = self.retrieve_and_insert_build(
                job_name, build_number)
            self.retrieve_and_insert_build_resultset(db_build_id)

    def retrieve_jenkins_results(self, job_names=None, pattern=None):
        """Retrieve builds and results either by name or matching a pattern."""
        if pattern is not None:
            all_job_names = self.jenkins.get_jobs_list()
            job_names = filter_job_names(all_job_names, pattern)
        for job_name in job_names:
            self.retrieve_data_for_job(job_name)


def _parse_arguments():
    parser = argparse.ArgumentParser(
        "Retrieve Jenkins build results, storing in a mongo db.")
    parser.add_argument(
        '-v',
        '--verbose',
        help='Verbose logging.',
        default=False,
        action='store_true',
    )
    parser.add_argument(
        '--vis-db',
        help='MongoDB collection in which to store data.',
        type=str,
        default='vis',
        required=False,
    )
    parser.add_argument(
        '--jenkins-url',
        help='Jenkins URL from which to collect data.',
        type=str,
        required=True,
    )
    parser.add_argument(
        '--jenkins-user',
        help='Jenkins username.',
        type=str,
        required=False,
    )
    parser.add_argument(
        '--jenkins-password',
        help='Jenkins API token for Jenkins user.',
        type=str,
        required=False,
    )
    parser.add_argument(
        '--mongo-host',
        help='MongoDB host IP.',
        type=str,
        default='127.0.0.1',
        required=False,
    )
    parser.add_argument(
        nargs='*',
        help='Job names from which to collect data.',
        type=str,
        dest='job_names',
    )
    parser.add_argument(
        '-p',
        '--pattern',
        help='Job name search pattern.',
        type=str,
    )
    args = parser.parse_args()
    if args.pattern and args.job_names:
        print("Please specify either job names or pattern.")
        parser.print_help()
        sys.exit(1)
    return args


def main():
    args = _parse_arguments()
    if args.verbose:
        logger.setLevel(logging.DEBUG)
    conn = pymongo.MongoClient(host=args.mongo_host)
    db = getattr(conn, args.vis_db)
    session = JenkinsMongoSession(
        db,
        args.jenkins_url,
        args.jenkins_user,
        args.jenkins_password,
    )
    session.retrieve_jenkins_results(
        args.job_names, args.pattern)


if __name__ == '__main__':
    sys.exit(main())
