#!/usr/bin/env python3
# UEQA VIS
# Copyright (C) 2014-2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest
import unittest.mock as mock

import qakit.vis.visualize as visualize


class StatusToFloatTestCase(unittest.TestCase):

    def test_passed(self):
        result = visualize.status_to_float('PASSED')
        self.assertEqual(1.0, result)

    def test_failed(self):
        result = visualize.status_to_float('FAILED')
        self.assertEqual(0.0, result)

    def test_fixed(self):
        result = visualize.status_to_float('FIXED')
        self.assertEqual(1.0, result)

    def test_regression(self):
        result = visualize.status_to_float('REGRESSION')
        self.assertEqual(0.0, result)

    def test_gibberish(self):
        result = visualize.status_to_float('GIBBERISH')
        self.assertEqual(0.0, result)


class ComputeColorTestCase(unittest.TestCase):

    def test_compute_color_pass(self):
        self.assertEqual('rgb(0, 255, 0)',
                         visualize.compute_color(1.0))

    def test_compute_color_fail(self):
        self.assertEqual('rgb(255, 0, 0)',
                         visualize.compute_color(0.0))

    def test_compute_color_none(self):
        self.assertEqual('rgb(255, 255, 255)',
                         visualize.compute_color(None))


class DbVisTestCase(unittest.TestCase):

    def setUp(self):
        super().setUp()
        self.db = mock.Mock(
            vis=mock.Mock())


@unittest.skip('Temporarily deprecated.')
class GetPassRateTestCase(unittest.TestCase):

    def test_pass_rate(self):
        pass_rate = 0.9
        pass_rate_dict = {'pass_rate': pass_rate}
        db = mock.Mock(
            dashboard_tests_pass_rates=mock.Mock(
                find_one=mock.Mock(
                    return_value=pass_rate_dict)))
        self.assertEqual(
            pass_rate,
            visualize.get_pass_rate(
                'fake_suite_name',
                'fake_build_name',
                db))

    def test_pass_rate_no_results(self):
        db = mock.Mock(
            dashboard_tests_pass_rates=mock.Mock(
                find_one=mock.Mock(
                    side_effect=TypeError)))
        self.assertEqual(
            None,
            visualize.get_pass_rate(
                'fake_suite_name',
                'fake_build_name',
                db))


class GetUrlPathFromTestNameTestCase(unittest.TestCase):

    def test_vanilla(self):
        result = visualize.get_url_path_from_test_name(
            'ubuntu_system_tests.tests.test_calls.CallsTestCase.'
            'test_receive_incoming_call_from_contact')
        self.assertEqual(
            'ubuntu_system_tests.tests.test_calls/CallsTestCase/'
            'test_receive_incoming_call_from_contact',
            result)

    def test_more_nesting(self):
        result = visualize.get_url_path_from_test_name(
            'ubuntu_system_tests.tests.webapps.test_here.HereTestCase.'
            'test_retrieve_location')
        self.assertEqual(
            'ubuntu_system_tests.tests.webapps.test_here/HereTestCase/'
            'test_retrieve_location',
            result)

    def test_odd_test_name_directs_to_build_test_result(self):
        result = visualize.get_url_path_from_test_name(
            'subunit.parser')
        self.assertEqual('', result)


class ComputePassRateFromListOfResultsTestCase(unittest.TestCase):

    def test_vanilla(self):
        result = visualize.compute_pass_rate_from_statuses(
            ['PASSED', 'PASSED'])
        self.assertEqual(1.0, result)

    def test_half_passed(self):
        result = visualize.compute_pass_rate_from_statuses(
            ['PASSED', 'FAILED'])
        self.assertEqual(0.5, result)


class SplitTestNameTestCase(unittest.TestCase):

    def test_vanilla(self):
        result = visualize.split_test_name(
            'foo.bar.FooTestCase.test_something')
        self.assertEqual(
            ('foo.bar', 'FooTestCase.test_something'),
            result,
        )

    def test_deeper_nesting(self):
        result = visualize.split_test_name(
            'foo.bar.baz.FooTestCase.test_something')
        self.assertEqual(
            ('foo.bar.baz', 'FooTestCase.test_something'),
            result,
        )

    def test_gibberish(self):
        result = visualize.split_test_name(
            'gibberish')
        self.assertEqual(
            ('gibberish', 'gibberish'),
            result,
        )

    def test_gibberish_with_dots(self):
        result = visualize.split_test_name(
            'gibberish.test_something')
        self.assertEqual(
            ('gibberish.test_something', 'gibberish.test_something'),
            result,
        )


class GetColorFromStatus(unittest.TestCase):

    def test_skipped(self):
        result = visualize.get_color_from_status('SKIPPED')
        self.assertEqual('white', result)

    def test_passed(self):
        result = visualize.get_color_from_status('PASSED')
        self.assertEqual('lime', result)

    def test_failed(self):
        result = visualize.get_color_from_status('FAILED')
        self.assertEqual('red', result)

    def test_regression(self):
        result = visualize.get_color_from_status('REGRESSION')
        self.assertEqual('red', result)

    def test_fixed(self):
        result = visualize.get_color_from_status('FIXED')
        self.assertEqual('lime', result)
