#!/usr/bin/env python3
# UEQA Vis
# Copyright (C) 2014-2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import argparse
import json
import math
import statistics
import sys

import pymongo

import jenkins_results


def get_color_from_status(status):
    """Return the color for the given cell.

    PASSED (or FIXED) = green,
    FAILED (or REGRESSED) = red,
    SKIPPED = white

    """
    red = 'red'
    white = 'white'
    # lime happens to help with trend color-calculations
    green = 'lime'
    return {
        'SKIPPED': white,
        'PASSED': green,
        'FIXED': green,
        'REGRESSION': red,
        'FAILED': red,
    }[status]


def compute_color(pass_rate):
    """Return a red-green color representing the given pass rate.

    Use a sqrt curve to highlight the difference between 0.0 and
    0.01: invert the value, apply sqrt, then invert the value
    again.

    """
    if pass_rate is None:
        # white indicates "no result"
        return 'rgb(255, 255, 255)'
    p = abs(
        1.0 - math.sqrt(
            abs(1.0 - float(pass_rate))
        )
    ) * 255.0
    return 'rgb({}, {}, 0)'.format(
        int(255 - p),
        int(p),
    )


def status_to_float(status):
    """Convert JUnit test status to float for averaging."""
    if status == 'PASSED' or status == 'FIXED':
        return 1.0
    else:
        return 0.0


def compute_pass_rate_from_statuses(results):
    """Compute pass rate from a list of JUnit test statuses."""
    float_results = [status_to_float(result) for result in results]
    try:
        return statistics.mean(float_results)
    except statistics.StatisticsError:
        return None


def compute_color_from_statuses(statuses):
    """Compute a red-green color from a list of JUnit test statuses."""
    return compute_color(compute_pass_rate_from_statuses(statuses))


def get_build_pass_rate(dao, job_name, build_number):
    """Calculate pass rate for a specified build in the given db."""
    results = dao.get_build_test_results(job_name, build_number)
    try:
        statuses = [result['status'] for result in results]
        return compute_pass_rate_from_statuses(statuses)
    except TypeError:
        # no result in db
        return None


def get_url_path_from_test_name(test_name):
    """Return url path of Jenkins report for test.

    Jenkins displays the result (incl. e.g. traceback) for a given
    test at specific URL path under the build URL.

    """
    split_test_name = test_name.split('.')
    if len(split_test_name) < 5:
        return ''
    try:
        url_path = '/'.join(
            ['.'.join(
                split_test_name[:-2]),
             split_test_name[-2],
             split_test_name[-1]])
    except IndexError:
        url_path = ''
    return url_path


def get_test_link_for_build(build_url, test_name):
    """Return a link to the test under the given Jenkins build."""
    link = get_url_path_from_test_name(test_name)
    return build_url + 'testReport/' + link


def get_cell_data(dao, job_name, build_number, test_name):
    """Return a dict representing the given test for rendering."""
    cell = {'test_name': test_name,
            'job_name': job_name,
            'build_number': build_number}
    test_result = dao.get_test_result(**cell)
    build = dao.get_build(job_name, build_number)
    if not test_result:
        return dict({'status': None, 'color': None, 'link': None}, **cell)
    cell['status'] = test_result['status']
    try:
        cell['color'] = get_color_from_status(test_result['status'])
    except TypeError:
        # no run for test, default to white
        cell['color'] = 'rgb{255, 255, 255}'
    cell['skipped'] = test_result['status'] == 'SKIPPED'
    cell['link'] = get_test_link_for_build(build['url'], test_name)
    return cell


def get_trend_cell(dao, job_name, test_name, limit=10):
    """Return a dict representing the pass rate trend."""
    cell = {'test_name': test_name,
            'job_name': job_name}
    test_results = dao.get_trailing_test_results(**cell)
    cell['color'] = compute_color_from_statuses(
        [test_result['status'] for test_result in test_results]
    )
    return cell


def split_test_name(test_name):
    """Return a tuple of (section, test_name) for given test name.

    Typically tests are named as

        foo.bar.baz.TestCaseClass.test_something

    so return a "section" as 'foo.bar.baz', test_name as
    TestCaseClass.test_something.

    Non-conforming test names (e.g. 'gibberish') are granted their own
    section names: ('gibberish', 'gibberish').

    """
    split_test_name = test_name.split('.')
    if len(split_test_name) <= 2:
        return (test_name, test_name)
    test_name = '.'.join(split_test_name[-2:])
    section_name = '.'.join(split_test_name[:-2])
    return (section_name, test_name)


def get_test_row(dao, job_name, test_name, build_numbers):
    """Return a dict of test results for rendering."""
    test_name_split = split_test_name(test_name)
    row = {
        'section': test_name_split[0],
        'test_name': test_name_split[1],
    }
    row['trend'] = get_trend_cell(dao, job_name, test_name)
    builds = []
    for build_number in build_numbers:
        builds.append(get_cell_data(
            dao, job_name, build_number, test_name))
    row['builds'] = builds
    return row


def get_report(
        dao,
        job_name):
    """Generate a JSON representation from db data."""
    build_numbers = dao.get_build_numbers(job_name)
    pass_rates = []
    for build_number in build_numbers:
        pass_rates.append(get_build_pass_rate(dao, job_name, build_number))

    report = {}
    report['header'] = ['Section', 'Test Name', 'Trend']

    # TODO: this may belong in the DataTables js, easier to compose here
    def get_pass_rate_build_number_header(pass_rate, build_number):
        try:
            # express pass_rate as a percentage
            return "{} ({:2.1f}%)".format(build_number, pass_rate*100)
        except TypeError:
            # when we encounter a None:
            return "{}".format(build_number)

    report['header'].extend([
        get_pass_rate_build_number_header(pass_rate, build_number)
        for build_number, pass_rate in
        zip(build_numbers, pass_rates)
    ])

    test_names = dao.get_test_names(job_name)
    data = []
    for test_name in test_names:
        data.append(
            get_test_row(
                dao,
                job_name,
                test_name,
                build_numbers,
            )
        )
    report['data'] = data
    return report


def write_json_report(
        dao,
        job_name,
        output_filepath=None):
    """Write a JSON report of test perf to the given filepath."""
    json_report = get_report(dao, job_name)
    with open(output_filepath, 'w') as f:
        f.write(json.dumps(json_report, indent=4))


def _parse_arguments():
    parser = argparse.ArgumentParser(
        "Write a JSON report of test perf.")
    parser.add_argument(
        help='Jenkins job to visualize.',
        type=str,
        dest='jenkins_job')
    parser.add_argument(
        '-o',
        help="Output filepath for JSON report.",
        type=str,
        dest='output_filepath',
        required=True)
    parser.add_argument(
        '--mongo-host',
        help='MongoDB host IP.',
        type=str,
        default='127.0.0.1',
        required=False)
    parser.add_argument(
        '--vis-db',
        help='MongoDB database from which to generate report.',
        type=str,
        default='vis',
        required=False)
    return parser.parse_args()


def main():
    args = _parse_arguments()
    conn = pymongo.MongoClient(host=args.mongo_host)
    db = getattr(conn, args.vis_db)
    dao = jenkins_results.JenkinsResults(db)
    return write_json_report(
        dao,
        args.jenkins_job,
        args.output_filepath,
    )


if __name__ == '__main__':
    sys.exit(main())
